import datetime
import os

from mopidy import settings
from mopidy.utils.path import mtime, uri_to_path
from mopidy.frontends.mpd import translator, protocol
from mopidy.models import Album, Artist, Playlist, Track

from tests import unittest


class TrackMpdFormatTest(unittest.TestCase):
    track = Track(
        uri=u'a uri',
        artists=[Artist(name=u'an artist')],
        name=u'a name',
        album=Album(name=u'an album', num_tracks=13,
            artists=[Artist(name=u'an other artist')]),
        track_no=7,
        date=datetime.date(1977, 1, 1),
        length=137000,
    )

    def setUp(self):
        settings.LOCAL_MUSIC_PATH = '/dir/subdir'
        mtime.set_fake_time(1234567)

    def tearDown(self):
        settings.runtime.clear()
        mtime.undo_fake()

    def test_track_to_mpd_format_for_empty_track(self):
        result = translator.track_to_mpd_format(Track())
        self.assert_(('file', '') in result)
        self.assert_(('Time', 0) in result)
        self.assert_(('Artist', '') in result)
        self.assert_(('Title', '') in result)
        self.assert_(('Album', '') in result)
        self.assert_(('Track', 0) in result)
        self.assert_(('Date', '') in result)
        self.assertEqual(len(result), 7)

    def test_track_to_mpd_format_with_position(self):
        result = translator.track_to_mpd_format(Track(), position=1)
        self.assert_(('Pos', 1) not in result)

    def test_track_to_mpd_format_with_cpid(self):
        result = translator.track_to_mpd_format(Track(), cpid=1)
        self.assert_(('Id', 1) not in result)

    def test_track_to_mpd_format_with_position_and_cpid(self):
        result = translator.track_to_mpd_format(Track(), position=1, cpid=2)
        self.assert_(('Pos', 1) in result)
        self.assert_(('Id', 2) in result)

    def test_track_to_mpd_format_for_nonempty_track(self):
        result = translator.track_to_mpd_format(
            self.track, position=9, cpid=122)
        self.assert_(('file', 'a uri') in result)
        self.assert_(('Time', 137) in result)
        self.assert_(('Artist', 'an artist') in result)
        self.assert_(('Title', 'a name') in result)
        self.assert_(('Album', 'an album') in result)
        self.assert_(('AlbumArtist', 'an other artist') in result)
        self.assert_(('Track', '7/13') in result)
        self.assert_(('Date', datetime.date(1977, 1, 1)) in result)
        self.assert_(('Pos', 9) in result)
        self.assert_(('Id', 122) in result)
        self.assertEqual(len(result), 10)

    def test_track_to_mpd_format_musicbrainz_trackid(self):
        track = self.track.copy(musicbrainz_id='foo')
        result = translator.track_to_mpd_format(track)
        self.assert_(('MUSICBRAINZ_TRACKID', 'foo') in result)

    def test_track_to_mpd_format_musicbrainz_albumid(self):
        album = self.track.album.copy(musicbrainz_id='foo')
        track = self.track.copy(album=album)
        result = translator.track_to_mpd_format(track)
        self.assert_(('MUSICBRAINZ_ALBUMID', 'foo') in result)

    def test_track_to_mpd_format_musicbrainz_albumid(self):
        artist = list(self.track.artists)[0].copy(musicbrainz_id='foo')
        album = self.track.album.copy(artists=[artist])
        track = self.track.copy(album=album)
        result = translator.track_to_mpd_format(track)
        self.assert_(('MUSICBRAINZ_ALBUMARTISTID', 'foo') in result)

    def test_track_to_mpd_format_musicbrainz_artistid(self):
        artist = list(self.track.artists)[0].copy(musicbrainz_id='foo')
        track = self.track.copy(artists=[artist])
        result = translator.track_to_mpd_format(track)
        self.assert_(('MUSICBRAINZ_ARTISTID', 'foo') in result)

    def test_artists_to_mpd_format(self):
        artists = [Artist(name=u'ABBA'), Artist(name=u'Beatles')]
        translated = translator.artists_to_mpd_format(artists)
        self.assertEqual(translated, u'ABBA, Beatles')

    def test_artists_to_mpd_format_artist_with_no_name(self):
        artists = [Artist(name=None)]
        translated = translator.artists_to_mpd_format(artists)
        self.assertEqual(translated, u'')


class PlaylistMpdFormatTest(unittest.TestCase):
    def test_mpd_format(self):
        playlist = Playlist(tracks=[
            Track(track_no=1), Track(track_no=2), Track(track_no=3)])
        result = translator.playlist_to_mpd_format(playlist)
        self.assertEqual(len(result), 3)

    def test_mpd_format_with_range(self):
        playlist = Playlist(tracks=[
            Track(track_no=1), Track(track_no=2), Track(track_no=3)])
        result = translator.playlist_to_mpd_format(playlist, 1, 2)
        self.assertEqual(len(result), 1)
        self.assertEqual(dict(result[0])['Track'], 2)


class TracksToTagCacheFormatTest(unittest.TestCase):
    def setUp(self):
        settings.LOCAL_MUSIC_PATH = '/dir/subdir'
        mtime.set_fake_time(1234567)

    def tearDown(self):
        settings.runtime.clear()
        mtime.undo_fake()

    def translate(self, track):
        folder = settings.LOCAL_MUSIC_PATH
        result = dict(translator.track_to_mpd_format(track))
        result['file'] = uri_to_path(result['file'])
        result['file'] = result['file'][len(folder)+1:]
        result['key'] = os.path.basename(result['file'])
        result['mtime'] = mtime('')
        return translator.order_mpd_track_info(result.items())

    def consume_headers(self, result):
        self.assertEqual(('info_begin',), result[0])
        self.assertEqual(('mpd_version', protocol.VERSION), result[1])
        self.assertEqual(('fs_charset', protocol.ENCODING), result[2])
        self.assertEqual(('info_end',), result[3])
        return result[4:]

    def consume_song_list(self, result):
        self.assertEqual(('songList begin',), result[0])
        for i, row in enumerate(result):
            if row == ('songList end',):
                return result[1:i], result[i+1:]
        self.fail("Couldn't find songList end in result")

    def consume_directory(self, result):
        self.assertEqual('directory', result[0][0])
        self.assertEqual(('mtime', mtime('.')), result[1])
        self.assertEqual(('begin', os.path.split(result[0][1])[1]), result[2])
        directory = result[2][1]
        for i, row in enumerate(result):
            if row == ('end', directory):
                return result[3:i], result[i+1:]
        self.fail("Couldn't find end %s in result" % directory)

    def test_empty_tag_cache_has_header(self):
        result = translator.tracks_to_tag_cache_format([])
        result = self.consume_headers(result)

    def test_empty_tag_cache_has_song_list(self):
        result = translator.tracks_to_tag_cache_format([])
        result = self.consume_headers(result)
        song_list, result = self.consume_song_list(result)

        self.assertEqual(len(song_list), 0)
        self.assertEqual(len(result), 0)

    def test_tag_cache_has_header(self):
        track = Track(uri='file:///dir/subdir/song.mp3')
        result = translator.tracks_to_tag_cache_format([track])
        result = self.consume_headers(result)

    def test_tag_cache_has_song_list(self):
        track = Track(uri='file:///dir/subdir/song.mp3')
        result = translator.tracks_to_tag_cache_format([track])
        result = self.consume_headers(result)
        song_list, result = self.consume_song_list(result)

        self.assert_(song_list)
        self.assertEqual(len(result), 0)

    def test_tag_cache_has_formated_track(self):
        track = Track(uri='file:///dir/subdir/song.mp3')
        formated = self.translate(track)
        result = translator.tracks_to_tag_cache_format([track])

        result = self.consume_headers(result)
        song_list, result = self.consume_song_list(result)

        self.assertEqual(song_list, formated)
        self.assertEqual(len(result), 0)

    def test_tag_cache_has_formated_track_with_key_and_mtime(self):
        track = Track(uri='file:///dir/subdir/song.mp3')
        formated = self.translate(track)
        result = translator.tracks_to_tag_cache_format([track])

        result = self.consume_headers(result)
        song_list, result = self.consume_song_list(result)

        self.assertEqual(song_list, formated)
        self.assertEqual(len(result), 0)

    def test_tag_cache_suports_directories(self):
        track = Track(uri='file:///dir/subdir/folder/song.mp3')
        formated = self.translate(track)
        result = translator.tracks_to_tag_cache_format([track])

        result = self.consume_headers(result)
        folder, result = self.consume_directory(result)
        song_list, result = self.consume_song_list(result)
        self.assertEqual(len(song_list), 0)
        self.assertEqual(len(result), 0)

        song_list, result = self.consume_song_list(folder)
        self.assertEqual(len(result), 0)
        self.assertEqual(song_list, formated)

    def test_tag_cache_diretory_header_is_right(self):
        track = Track(uri='file:///dir/subdir/folder/sub/song.mp3')
        result = translator.tracks_to_tag_cache_format([track])

        result = self.consume_headers(result)
        folder, result = self.consume_directory(result)

        self.assertEqual(('directory', 'folder/sub'), folder[0])
        self.assertEqual(('mtime', mtime('.')), folder[1])
        self.assertEqual(('begin', 'sub'), folder[2])

    def test_tag_cache_suports_sub_directories(self):
        track = Track(uri='file:///dir/subdir/folder/sub/song.mp3')
        formated = self.translate(track)
        result = translator.tracks_to_tag_cache_format([track])

        result = self.consume_headers(result)

        folder, result = self.consume_directory(result)
        song_list, result = self.consume_song_list(result)
        self.assertEqual(len(song_list), 0)
        self.assertEqual(len(result), 0)

        folder, result = self.consume_directory(folder)
        song_list, result = self.consume_song_list(result)
        self.assertEqual(len(result), 0)
        self.assertEqual(len(song_list), 0)

        song_list, result = self.consume_song_list(folder)
        self.assertEqual(len(result), 0)
        self.assertEqual(song_list, formated)

    def test_tag_cache_supports_multiple_tracks(self):
        tracks = [
            Track(uri='file:///dir/subdir/song1.mp3'),
            Track(uri='file:///dir/subdir/song2.mp3'),
        ]

        formated = []
        formated.extend(self.translate(tracks[0]))
        formated.extend(self.translate(tracks[1]))

        result = translator.tracks_to_tag_cache_format(tracks)

        result = self.consume_headers(result)
        song_list, result = self.consume_song_list(result)

        self.assertEqual(song_list, formated)
        self.assertEqual(len(result), 0)

    def test_tag_cache_supports_multiple_tracks_in_dirs(self):
        tracks = [
            Track(uri='file:///dir/subdir/song1.mp3'),
            Track(uri='file:///dir/subdir/folder/song2.mp3'),
        ]

        formated = []
        formated.append(self.translate(tracks[0]))
        formated.append(self.translate(tracks[1]))

        result = translator.tracks_to_tag_cache_format(tracks)

        result = self.consume_headers(result)
        folder, result = self.consume_directory(result)
        song_list, song_result = self.consume_song_list(folder)

        self.assertEqual(song_list, formated[1])
        self.assertEqual(len(song_result), 0)

        song_list, result = self.consume_song_list(result)
        self.assertEqual(len(result), 0)
        self.assertEqual(song_list, formated[0])


class TracksToDirectoryTreeTest(unittest.TestCase):
    def setUp(self):
        settings.LOCAL_MUSIC_PATH = '/root/'

    def tearDown(self):
        settings.runtime.clear()

    def test_no_tracks_gives_emtpy_tree(self):
        tree = translator.tracks_to_directory_tree([])
        self.assertEqual(tree, ({}, []))

    def test_top_level_files(self):
        tracks = [
            Track(uri='file:///root/file1.mp3'),
            Track(uri='file:///root/file2.mp3'),
            Track(uri='file:///root/file3.mp3'),
        ]
        tree = translator.tracks_to_directory_tree(tracks)
        self.assertEqual(tree, ({}, tracks))

    def test_single_file_in_subdir(self):
        tracks = [Track(uri='file:///root/dir/file1.mp3')]
        tree = translator.tracks_to_directory_tree(tracks)
        expected = ({'dir': ({}, tracks)}, [])
        self.assertEqual(tree, expected)

    def test_single_file_in_sub_subdir(self):
        tracks = [Track(uri='file:///root/dir1/dir2/file1.mp3')]
        tree = translator.tracks_to_directory_tree(tracks)
        expected = ({'dir1': ({'dir1/dir2': ({}, tracks)}, [])}, [])
        self.assertEqual(tree, expected)

    def test_complex_file_structure(self):
        tracks = [
            Track(uri='file:///root/file1.mp3'),
            Track(uri='file:///root/dir1/file2.mp3'),
            Track(uri='file:///root/dir1/file3.mp3'),
            Track(uri='file:///root/dir2/file4.mp3'),
            Track(uri='file:///root/dir2/sub/file5.mp3'),
        ]
        tree = translator.tracks_to_directory_tree(tracks)
        expected = (
            {
                'dir1': ({}, [tracks[1], tracks[2]]),
                'dir2': (
                    {
                        'dir2/sub': ({}, [tracks[4]])
                    },
                    [tracks[3]]
                ),
            },
            [tracks[0]]
        )
        self.assertEqual(tree, expected)



import os
import sys

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from mopidy import settings

# Nuke any local settings to ensure same test env all over
settings.local.clear()


def path_to_data_dir(name):
    path = os.path.dirname(__file__)
    path = os.path.join(path, 'data')
    path = os.path.abspath(path)
    return os.path.join(path, name)


class IsA(object):
    def __init__(self, klass):
        self.klass = klass

    def __eq__(self, rhs):
        try:
            return isinstance(rhs, self.klass)
        except TypeError:
            return type(rhs) == type(self.klass)

    def __ne__(self, rhs):
        return not self.__eq__(rhs)

    def __repr__(self):
        return str(self.klass)


any_int = IsA(int)
any_str = IsA(str)
any_unicode = IsA(unicode)



# Absolute import needed to import ~/.mopidy/settings.py and not ourselves
from __future__ import absolute_import
from copy import copy
import getpass
import glib
import logging
import os
from pprint import pformat
import sys

from mopidy import SettingsError, SETTINGS_PATH, SETTINGS_FILE
from mopidy.utils.log import indent

logger = logging.getLogger('mopidy.utils.settings')

class SettingsProxy(object):
    def __init__(self, default_settings_module):
        self.default = self._get_settings_dict_from_module(
            default_settings_module)
        self.local = self._get_local_settings()
        self.runtime = {}

    def _get_local_settings(self):
        if not os.path.isfile(SETTINGS_FILE):
            return {}
        sys.path.insert(0, SETTINGS_PATH)
        # pylint: disable = F0401
        import settings as local_settings_module
        # pylint: enable = F0401
        return self._get_settings_dict_from_module(local_settings_module)

    def _get_settings_dict_from_module(self, module):
        settings = filter(lambda (key, value): self._is_setting(key),
            module.__dict__.iteritems())
        return dict(settings)

    def _is_setting(self, name):
        return name.isupper()

    @property
    def current(self):
        current = copy(self.default)
        current.update(self.local)
        current.update(self.runtime)
        return current

    def __getattr__(self, attr):
        if not self._is_setting(attr):
            return
        if attr not in self.current:
            raise SettingsError(u'Setting "%s" is not set.' % attr)
        value = self.current[attr]
        if isinstance(value, basestring) and len(value) == 0:
            raise SettingsError(u'Setting "%s" is empty.' % attr)
        if not value:
            return value
        if attr.endswith('_PATH') or attr.endswith('_FILE'):
            value = os.path.expanduser(value)
            value = os.path.abspath(value)
        return value

    def __setattr__(self, attr, value):
        if self._is_setting(attr):
            self.runtime[attr] = value
        else:
            super(SettingsProxy, self).__setattr__(attr, value)

    def validate(self, interactive):
        if interactive:
            self._read_missing_settings_from_stdin(self.current, self.runtime)
        if self.get_errors():
            logger.error(u'Settings validation errors: %s',
                indent(self.get_errors_as_string()))
            raise SettingsError(u'Settings validation failed.')

    def _read_missing_settings_from_stdin(self, current, runtime):
        for setting, value in sorted(current.iteritems()):
            if isinstance(value, basestring) and len(value) == 0:
                runtime[setting] = self._read_from_stdin(setting + u': ')

    def _read_from_stdin(self, prompt):
        if u'_PASSWORD' in prompt:
            return (getpass.getpass(prompt)
                .decode(sys.stdin.encoding, 'ignore'))
        else:
            sys.stdout.write(prompt)
            return (sys.stdin.readline().strip()
                .decode(sys.stdin.encoding, 'ignore'))

    def get_errors(self):
        return validate_settings(self.default, self.local)

    def get_errors_as_string(self):
        lines = []
        for (setting, error) in self.get_errors().iteritems():
            lines.append(u'%s: %s' % (setting, error))
        return '\n'.join(lines)


def validate_settings(defaults, settings):
    """
    Checks the settings for both errors like misspellings and against a set of
    rules for renamed settings, etc.

    Returns of setting names with associated errors.

    :param defaults: Mopidy's default settings
    :type defaults: dict
    :param settings: the user's local settings
    :type settings: dict
    :rtype: dict
    """
    errors = {}

    changed = {
        'DUMP_LOG_FILENAME': 'DEBUG_LOG_FILENAME',
        'DUMP_LOG_FORMAT': 'DEBUG_LOG_FORMAT',
        'FRONTEND': 'FRONTENDS',
        'GSTREAMER_AUDIO_SINK': 'CUSTOM_OUTPUT',
        'LOCAL_MUSIC_FOLDER': 'LOCAL_MUSIC_PATH',
        'LOCAL_OUTPUT_OVERRIDE': 'CUSTOM_OUTPUT',
        'LOCAL_PLAYLIST_FOLDER': 'LOCAL_PLAYLIST_PATH',
        'LOCAL_TAG_CACHE': 'LOCAL_TAG_CACHE_FILE',
        'OUTPUT': None,
        'SERVER': None,
        'SERVER_HOSTNAME': 'MPD_SERVER_HOSTNAME',
        'SERVER_PORT': 'MPD_SERVER_PORT',
        'SPOTIFY_HIGH_BITRATE': 'SPOTIFY_BITRATE',
        'SPOTIFY_LIB_APPKEY': None,
        'SPOTIFY_LIB_CACHE': 'SPOTIFY_CACHE_PATH',
    }

    for setting, value in settings.iteritems():
        if setting in changed:
            if changed[setting] is None:
                errors[setting] = u'Deprecated setting. It may be removed.'
            else:
                errors[setting] = u'Deprecated setting. Use %s.' % (
                    changed[setting],)
            continue

        if setting == 'BACKENDS':
            if 'mopidy.backends.despotify.DespotifyBackend' in value:
                errors[setting] = (u'Deprecated setting value. ' +
                    '"mopidy.backends.despotify.DespotifyBackend" is no ' +
                    'longer available.')
                continue

        if setting == 'SPOTIFY_BITRATE':
            if value not in (96, 160, 320):
                errors[setting] = (u'Unavailable Spotify bitrate. ' +
                    u'Available bitrates are 96, 160, and 320.')

        if setting not in defaults:
            errors[setting] = u'Unknown setting. Is it misspelled?'
            continue

    return errors

def list_settings_optparse_callback(*args):
    """
    Prints a list of all settings.

    Called by optparse when Mopidy is run with the :option:`--list-settings`
    option.
    """
    from mopidy import settings
    print format_settings_list(settings)
    sys.exit(0)

def format_settings_list(settings):
    errors = settings.get_errors()
    lines = []
    for (key, value) in sorted(settings.current.iteritems()):
        default_value = settings.default.get(key)
        masked_value = mask_value_if_secret(key, value)
        lines.append(u'%s: %s' % (key, indent(pformat(masked_value), places=2)))
        if value != default_value and default_value is not None:
            lines.append(u'  Default: %s' %
                indent(pformat(default_value), places=4))
        if errors.get(key) is not None:
            lines.append(u'  Error: %s' % errors[key])
    return '\n'.join(lines)

def mask_value_if_secret(key, value):
    if key.endswith('PASSWORD') and value:
        return u'********'
    else:
        return value



import pygst
pygst.require('0.10')
import gst

import logging

from pykka.actor import ThreadingActor
from pykka.registry import ActorRegistry

from mopidy import settings
from mopidy.utils import get_class
from mopidy.backends.base import Backend

logger = logging.getLogger('mopidy.gstreamer')

default_caps = gst.Caps("""
    audio/x-raw-int,
    endianness=(int)1234,
    channels=(int)2,
    width=(int)16,
    depth=(int)16,
    signed=(boolean)true,
    rate=(int)44100""")


class GStreamer(ThreadingActor):
    """
    Audio output through `GStreamer <http://gstreamer.freedesktop.org/>`_.

    **Settings:**

    - :attr:`mopidy.settings.OUTPUTS`

    """

    def __init__(self):
        self._pipeline = None
        self._source = None
        self._tee = None
        self._uridecodebin = None
        self._volume = None
        self._outputs = []
        self._handlers = {}

    def on_start(self):
        self._setup_pipeline()
        self._setup_outputs()
        self._setup_message_processor()

    def _setup_pipeline(self):
        description = ' ! '.join([
            'uridecodebin name=uri',
            'audioconvert name=convert',
            'volume name=volume',
            'tee name=tee'])

        logger.debug(u'Setting up base GStreamer pipeline: %s', description)

        self._pipeline = gst.parse_launch(description)
        self._tee = self._pipeline.get_by_name('tee')
        self._volume = self._pipeline.get_by_name('volume')
        self._uridecodebin = self._pipeline.get_by_name('uri')

        self._uridecodebin.connect('notify::source', self._on_new_source)
        self._uridecodebin.connect('pad-added', self._on_new_pad,
            self._pipeline.get_by_name('convert').get_pad('sink'))

    def _setup_outputs(self):
        for output in settings.OUTPUTS:
            get_class(output)(self).connect()

    def _setup_message_processor(self):
        bus = self._pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self._on_message)

    def _on_new_source(self, element, pad):
        self._source = element.get_property('source')
        try:
            self._source.set_property('caps', default_caps)
        except TypeError:
            pass

    def _on_new_pad(self, source, pad, target_pad):
        if not pad.is_linked():
            pad.link(target_pad)

    def _on_message(self, bus, message):
        if message.src in self._handlers:
            if self._handlers[message.src](message):
                return # Message was handeled by output

        if message.type == gst.MESSAGE_EOS:
            logger.debug(u'GStreamer signalled end-of-stream. '
                'Telling backend ...')
            self._get_backend().playback.on_end_of_track()
        elif message.type == gst.MESSAGE_ERROR:
            error, debug = message.parse_error()
            logger.error(u'%s %s', error, debug)
            self.stop_playback()
        elif message.type == gst.MESSAGE_WARNING:
            error, debug = message.parse_warning()
            logger.warning(u'%s %s', error, debug)

    def _get_backend(self):
        backend_refs = ActorRegistry.get_by_class(Backend)
        assert len(backend_refs) == 1, 'Expected exactly one running backend.'
        return backend_refs[0].proxy()

    def set_uri(self, uri):
        """
        Set URI of audio to be played.

        You *MUST* call :meth:`prepare_change` before calling this method.

        :param uri: the URI to play
        :type uri: string
        """
        self._uridecodebin.set_property('uri', uri)

    def emit_data(self, capabilities, data):
        """
        Call this to deliver raw audio data to be played.

        :param capabilities: a GStreamer capabilities string
        :type capabilities: string
        :param data: raw audio data to be played
        """
        caps = gst.caps_from_string(capabilities)
        buffer_ = gst.Buffer(buffer(data))
        buffer_.set_caps(caps)
        self._source.set_property('caps', caps)
        self._source.emit('push-buffer', buffer_)

    def emit_end_of_stream(self):
        """
        Put an end-of-stream token on the pipeline. This is typically used in
        combination with :meth:`emit_data`.

        We will get a GStreamer message when the stream playback reaches the
        token, and can then do any end-of-stream related tasks.
        """
        self._source.emit('end-of-stream')

    def get_position(self):
        """
        Get position in milliseconds.

        :rtype: int
        """
        if self._pipeline.get_state()[1] == gst.STATE_NULL:
            return 0
        try:
            position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
            return position // gst.MSECOND
        except gst.QueryError, e:
            logger.error('time_position failed: %s', e)
            return 0

    def set_position(self, position):
        """
        Set position in milliseconds.

        :param position: the position in milliseconds
        :type volume: int
        :rtype: :class:`True` if successful, else :class:`False`
        """
        self._pipeline.get_state() # block until state changes are done
        handeled = self._pipeline.seek_simple(gst.Format(gst.FORMAT_TIME),
            gst.SEEK_FLAG_FLUSH, position * gst.MSECOND)
        self._pipeline.get_state() # block until seek is done
        return handeled

    def start_playback(self):
        """
        Notify GStreamer that it should start playback.

        :rtype: :class:`True` if successfull, else :class:`False`
        """
        return self._set_state(gst.STATE_PLAYING)

    def pause_playback(self):
        """
        Notify GStreamer that it should pause playback.

        :rtype: :class:`True` if successfull, else :class:`False`
        """
        return self._set_state(gst.STATE_PAUSED)

    def prepare_change(self):
        """
        Notify GStreamer that we are about to change state of playback.

        This function *MUST* be called before changing URIs or doing
        changes like updating data that is being pushed. The reason for this
        is that GStreamer will reset all its state when it changes to
        :attr:`gst.STATE_READY`.
        """
        return self._set_state(gst.STATE_READY)

    def stop_playback(self):
        """
        Notify GStreamer that is should stop playback.

        :rtype: :class:`True` if successfull, else :class:`False`
        """
        return self._set_state(gst.STATE_NULL)

    def _set_state(self, state):
        """
        Internal method for setting the raw GStreamer state.

        .. digraph:: gst_state_transitions

            graph [rankdir="LR"];
            node [fontsize=10];

            "NULL" -> "READY"
            "PAUSED" -> "PLAYING"
            "PAUSED" -> "READY"
            "PLAYING" -> "PAUSED"
            "READY" -> "NULL"
            "READY" -> "PAUSED"

        :param state: State to set pipeline to. One of: `gst.STATE_NULL`,
            `gst.STATE_READY`, `gst.STATE_PAUSED` and `gst.STATE_PLAYING`.
        :type state: :class:`gst.State`
        :rtype: :class:`True` if successfull, else :class:`False`
        """
        result = self._pipeline.set_state(state)
        if result == gst.STATE_CHANGE_FAILURE:
            logger.warning('Setting GStreamer state to %s: failed',
                state.value_name)
            return False
        elif result == gst.STATE_CHANGE_ASYNC:
            logger.debug('Setting GStreamer state to %s: async',
                state.value_name)
            return True
        else:
            logger.debug('Setting GStreamer state to %s: OK',
                state.value_name)
            return True

    def get_volume(self):
        """
        Get volume level of the GStreamer software mixer.

        :rtype: int in range [0..100]
        """
        return int(self._volume.get_property('volume') * 100)

    def set_volume(self, volume):
        """
        Set volume level of the GStreamer software mixer.

        :param volume: the volume in the range [0..100]
        :type volume: int
        :rtype: :class:`True` if successful, else :class:`False`
        """
        self._volume.set_property('volume', volume / 100.0)
        return True

    def set_metadata(self, track):
        """
        Set track metadata for currently playing song.

        Only needs to be called by sources such as `appsrc` which do not
        already inject tags in pipeline, e.g. when using :meth:`emit_data` to
        deliver raw audio data to GStreamer.

        :param track: the current track
        :type track: :class:`mopidy.modes.Track`
        """
        taglist = gst.TagList()
        artists = [a for a in (track.artists or []) if a.name]

        # Default to blank data to trick shoutcast into clearing any previous
        # values it might have.
        taglist[gst.TAG_ARTIST] = u' '
        taglist[gst.TAG_TITLE] = u' '
        taglist[gst.TAG_ALBUM] = u' '

        if artists:
            taglist[gst.TAG_ARTIST] = u', '.join([a.name for a in artists])

        if track.name:
            taglist[gst.TAG_TITLE] = track.name

        if track.album and track.album.name:
            taglist[gst.TAG_ALBUM] = track.album.name

        event = gst.event_new_tag(taglist)
        self._pipeline.send_event(event)

    def connect_output(self, output):
        """
        Connect output to pipeline.

        :param output: output to connect to the pipeline
        :type output: :class:`gst.Bin`
        """
        self._pipeline.add(output)
        output.sync_state_with_parent() # Required to add to running pipe
        gst.element_link_many(self._tee, output)
        self._outputs.append(output)
        logger.debug('GStreamer added %s', output.get_name())

    def list_outputs(self):
        """
        Get list with the name of all active outputs.

        :rtype: list of strings
        """
        return [output.get_name() for output in self._outputs]

    def remove_output(self, output):
        """
        Remove output from our pipeline.

        :param output: output to remove from the pipeline
        :type output: :class:`gst.Bin`
        """
        if output not in self._outputs:
            raise LookupError('Ouput %s not present in pipeline'
                % output.get_name)
        teesrc = output.get_pad('sink').get_peer()
        handler = teesrc.add_event_probe(self._handle_event_probe)

        struct = gst.Structure('mopidy-unlink-tee')
        struct.set_value('handler', handler)

        event = gst.event_new_custom(gst.EVENT_CUSTOM_DOWNSTREAM, struct)
        self._tee.send_event(event)

    def _handle_event_probe(self, teesrc, event):
        if event.type == gst.EVENT_CUSTOM_DOWNSTREAM and event.has_name('mopidy-unlink-tee'):
            data = self._get_structure_data(event.get_structure())

            output = teesrc.get_peer().get_parent()

            teesrc.unlink(teesrc.get_peer())
            teesrc.remove_event_probe(data['handler'])

            output.set_state(gst.STATE_NULL)
            self._pipeline.remove(output)

            logger.warning('Removed %s', output.get_name())
            return False
        return True

    def _get_structure_data(self, struct):
        # Ugly hack to get around missing get_value in pygst bindings :/
        data = {}
        def get_data(key, value):
            data[key] = value
        struct.foreach(get_data)
        return data

    def connect_message_handler(self, element, handler):
        """
        Attach custom message handler for given element.

        Hook to allow outputs (or other code) to register custom message
        handlers for all messages coming from the element in question.

        In the case of outputs, :meth:`mopidy.outputs.BaseOutput.on_connect`
        should be used to attach such handlers and care should be taken to
        remove them in :meth:`mopidy.outputs.BaseOutput.on_remove` using
        :meth:`remove_message_handler`.

        The handler callback will only be given the message in question, and
        is free to ignore the message. However, if the handler wants to prevent
        the default handling of the message it should return :class:`True`
        indicating that the message has been handled.

        Note that there can only be one handler per element.

        :param element: element to watch messages from
        :type element: :class:`gst.Element`
        :param handler: callable that takes :class:`gst.Message` and returns
            :class:`True` if the message has been handeled
        :type handler: callable
        """
        self._handlers[element] = handler

    def remove_message_handler(self, element):
        """
        Remove custom message handler.

        :param element: element to remove message handling from.
        :type element: :class:`gst.Element`
        """
        self._handlers.pop(element, None)



import pykka.future

from mopidy.backends.base import PlaybackController
from mopidy.frontends.mpd.protocol import handle_request
from mopidy.frontends.mpd.exceptions import MpdNotImplemented

#: Subsystems that can be registered with idle command.
SUBSYSTEMS = ['database', 'mixer', 'options', 'output',
    'player', 'playlist', 'stored_playlist', 'update', ]

@handle_request(r'^clearerror$')
def clearerror(context):
    """
    *musicpd.org, status section:*

        ``clearerror``

        Clears the current error message in status (this is also
        accomplished by any command that starts playback).
    """
    raise MpdNotImplemented # TODO

@handle_request(r'^currentsong$')
def currentsong(context):
    """
    *musicpd.org, status section:*

        ``currentsong``

        Displays the song info of the current song (same song that is
        identified in status).
    """
    current_cp_track = context.backend.playback.current_cp_track.get()
    if current_cp_track is not None:
        return current_cp_track.track.mpd_format(
            position=context.backend.playback.current_playlist_position.get(),
            cpid=current_cp_track.cpid)

@handle_request(r'^idle$')
@handle_request(r'^idle (?P<subsystems>.+)$')
def idle(context, subsystems=None):
    """
    *musicpd.org, status section:*

        ``idle [SUBSYSTEMS...]``

        Waits until there is a noteworthy change in one or more of MPD's
        subsystems. As soon as there is one, it lists all changed systems
        in a line in the format ``changed: SUBSYSTEM``, where ``SUBSYSTEM``
        is one of the following:

        - ``database``: the song database has been modified after update.
        - ``update``: a database update has started or finished. If the
          database was modified during the update, the database event is
          also emitted.
        - ``stored_playlist``: a stored playlist has been modified,
          renamed, created or deleted
        - ``playlist``: the current playlist has been modified
        - ``player``: the player has been started, stopped or seeked
        - ``mixer``: the volume has been changed
        - ``output``: an audio output has been enabled or disabled
        - ``options``: options like repeat, random, crossfade, replay gain

        While a client is waiting for idle results, the server disables
        timeouts, allowing a client to wait for events as long as MPD runs.
        The idle command can be canceled by sending the command ``noidle``
        (no other commands are allowed). MPD will then leave idle mode and
        print results immediately; might be empty at this time.

        If the optional ``SUBSYSTEMS`` argument is used, MPD will only send
        notifications when something changed in one of the specified
        subsystems.
    """

    if subsystems:
        subsystems = subsystems.split()
    else:
        subsystems = SUBSYSTEMS

    for subsystem in subsystems:
        context.subscriptions.add(subsystem)

    active = context.subscriptions.intersection(context.events)
    if not active:
        context.session.prevent_timeout = True
        return

    response = []
    context.events = set()
    context.subscriptions = set()

    for subsystem in active:
        response.append(u'changed: %s' % subsystem)
    return response

@handle_request(r'^noidle$')
def noidle(context):
    """See :meth:`_status_idle`."""
    if not context.subscriptions:
        return
    context.subscriptions = set()
    context.events = set()
    context.session.prevent_timeout = False

@handle_request(r'^stats$')
def stats(context):
    """
    *musicpd.org, status section:*

        ``stats``

        Displays statistics.

        - ``artists``: number of artists
        - ``songs``: number of albums
        - ``uptime``: daemon uptime in seconds
        - ``db_playtime``: sum of all song times in the db
        - ``db_update``: last db update in UNIX time
        - ``playtime``: time length of music played
    """
    return {
        'artists': 0, # TODO
        'albums': 0, # TODO
        'songs': 0, # TODO
        'uptime': 0, # TODO
        'db_playtime': 0, # TODO
        'db_update': 0, # TODO
        'playtime': 0, # TODO
    }

@handle_request(r'^status$')
def status(context):
    """
    *musicpd.org, status section:*

        ``status``

        Reports the current status of the player and the volume level.

        - ``volume``: 0-100
        - ``repeat``: 0 or 1
        - ``single``: 0 or 1
        - ``consume``: 0 or 1
        - ``playlist``: 31-bit unsigned integer, the playlist version
          number
        - ``playlistlength``: integer, the length of the playlist
        - ``state``: play, stop, or pause
        - ``song``: playlist song number of the current song stopped on or
          playing
        - ``songid``: playlist songid of the current song stopped on or
          playing
        - ``nextsong``: playlist song number of the next song to be played
        - ``nextsongid``: playlist songid of the next song to be played
        - ``time``: total time elapsed (of current playing/paused song)
        - ``elapsed``: Total time elapsed within the current song, but with
          higher resolution. 
        - ``bitrate``: instantaneous bitrate in kbps
        - ``xfade``: crossfade in seconds
        - ``audio``: sampleRate``:bits``:channels
        - ``updatings_db``: job id
        - ``error``: if there is an error, returns message here

    *Clarifications based on experience implementing*
        - ``volume``: can also be -1 if no output is set.
        - ``elapsed``: Higher resolution means time in seconds with three
          decimal places for millisecond precision.
    """
    futures = {
        'current_playlist.tracks': context.backend.current_playlist.tracks,
        'current_playlist.version': context.backend.current_playlist.version,
        'mixer.volume': context.mixer.volume,
        'playback.consume': context.backend.playback.consume,
        'playback.random': context.backend.playback.random,
        'playback.repeat': context.backend.playback.repeat,
        'playback.single': context.backend.playback.single,
        'playback.state': context.backend.playback.state,
        'playback.current_cp_track': context.backend.playback.current_cp_track,
        'playback.current_playlist_position':
            context.backend.playback.current_playlist_position,
        'playback.time_position': context.backend.playback.time_position,
    }
    pykka.future.get_all(futures.values())
    result = [
        ('volume', _status_volume(futures)),
        ('repeat', _status_repeat(futures)),
        ('random', _status_random(futures)),
        ('single', _status_single(futures)),
        ('consume', _status_consume(futures)),
        ('playlist', _status_playlist_version(futures)),
        ('playlistlength', _status_playlist_length(futures)),
        ('xfade', _status_xfade(futures)),
        ('state', _status_state(futures)),
    ]
    if futures['playback.current_cp_track'].get() is not None:
        result.append(('song', _status_songpos(futures)))
        result.append(('songid', _status_songid(futures)))
    if futures['playback.state'].get() in (PlaybackController.PLAYING,
            PlaybackController.PAUSED):
        result.append(('time', _status_time(futures)))
        result.append(('elapsed', _status_time_elapsed(futures)))
        result.append(('bitrate', _status_bitrate(futures)))
    return result

def _status_bitrate(futures):
    current_cp_track = futures['playback.current_cp_track'].get()
    if current_cp_track is not None:
        return current_cp_track.track.bitrate

def _status_consume(futures):
    if futures['playback.consume'].get():
        return 1
    else:
        return 0

def _status_playlist_length(futures):
    return len(futures['current_playlist.tracks'].get())

def _status_playlist_version(futures):
    return futures['current_playlist.version'].get()

def _status_random(futures):
    return int(futures['playback.random'].get())

def _status_repeat(futures):
    return int(futures['playback.repeat'].get())

def _status_single(futures):
    return int(futures['playback.single'].get())

def _status_songid(futures):
    current_cp_track = futures['playback.current_cp_track'].get()
    if current_cp_track is not None:
        return current_cp_track.cpid
    else:
        return _status_songpos(futures)

def _status_songpos(futures):
    return futures['playback.current_playlist_position'].get()

def _status_state(futures):
    state = futures['playback.state'].get()
    if state == PlaybackController.PLAYING:
        return u'play'
    elif state == PlaybackController.STOPPED:
        return u'stop'
    elif state == PlaybackController.PAUSED:
        return u'pause'

def _status_time(futures):
    return u'%d:%d' % (futures['playback.time_position'].get() // 1000,
        _status_time_total(futures) // 1000)

def _status_time_elapsed(futures):
    return u'%.3f' % (futures['playback.time_position'].get() / 1000.0)

def _status_time_total(futures):
    current_cp_track = futures['playback.current_cp_track'].get()
    if current_cp_track is None:
        return 0
    elif current_cp_track.track.length is None:
        return 0
    else:
        return current_cp_track.track.length

def _status_volume(futures):
    volume = futures['mixer.volume'].get()
    if volume is not None:
        return volume
    else:
        return 0

def _status_xfade(futures):
    return 0 # Not supported



import sys

from mopidy import settings
from mopidy.gstreamer import GStreamer
from mopidy.utils.path import path_to_uri

from tests import unittest, path_to_data_dir

# TODO BaseOutputTest?


@unittest.skipIf(sys.platform == 'win32',
    'Our Windows build server does not support GStreamer yet')
class GStreamerTest(unittest.TestCase):
    def setUp(self):
        settings.BACKENDS = ('mopidy.backends.local.LocalBackend',)
        self.song_uri = path_to_uri(path_to_data_dir('song1.wav'))
        self.gstreamer = GStreamer()
        self.gstreamer.on_start()

    def prepare_uri(self, uri):
        self.gstreamer.prepare_change()
        self.gstreamer.set_uri(uri)

    def tearDown(self):
        settings.runtime.clear()

    def test_start_playback_existing_file(self):
        self.prepare_uri(self.song_uri)
        self.assertTrue(self.gstreamer.start_playback())

    def test_start_playback_non_existing_file(self):
        self.prepare_uri(self.song_uri + 'bogus')
        self.assertFalse(self.gstreamer.start_playback())

    def test_pause_playback_while_playing(self):
        self.prepare_uri(self.song_uri)
        self.gstreamer.start_playback()
        self.assertTrue(self.gstreamer.pause_playback())

    def test_stop_playback_while_playing(self):
        self.prepare_uri(self.song_uri)
        self.gstreamer.start_playback()
        self.assertTrue(self.gstreamer.stop_playback())

    @unittest.SkipTest
    def test_deliver_data(self):
        pass # TODO

    @unittest.SkipTest
    def test_end_of_data_stream(self):
        pass # TODO

    def test_default_get_volume_result(self):
        self.assertEqual(100, self.gstreamer.get_volume())

    def test_set_volume(self):
        self.assertTrue(self.gstreamer.set_volume(50))
        self.assertEqual(50, self.gstreamer.get_volume())

    def test_set_volume_to_zero(self):
        self.assertTrue(self.gstreamer.set_volume(0))
        self.assertEqual(0, self.gstreamer.get_volume())

    def test_set_volume_to_one_hundred(self):
        self.assertTrue(self.gstreamer.set_volume(100))
        self.assertEqual(100, self.gstreamer.get_volume())

    @unittest.SkipTest
    def test_set_state_encapsulation(self):
        pass # TODO

    @unittest.SkipTest
    def test_set_position(self):
        pass # TODO



"""
Available settings and their default values.

.. warning::

    Do *not* change settings directly in :mod:`mopidy.settings`. Instead, add a
    file called ``~/.config/mopidy/settings.py`` and redefine settings there.
"""

#: List of playback backends to use. See :mod:`mopidy.backends` for all
#: available backends.
#:
#: Default::
#:
#:     BACKENDS = (u'mopidy.backends.spotify.SpotifyBackend',)
#:
#: .. note::
#:     Currently only the first backend in the list is used.
BACKENDS = (
    u'mopidy.backends.spotify.SpotifyBackend',
)

#: The log format used for informational logging.
#:
#: See http://docs.python.org/library/logging.html#formatter-objects for
#: details on the format.
CONSOLE_LOG_FORMAT = u'%(levelname)-8s %(message)s'

#: Which GStreamer bin description to use in
#: :class:`mopidy.outputs.custom.CustomOutput`.
#:
#: Default::
#:
#:     CUSTOM_OUTPUT = u'fakesink'
CUSTOM_OUTPUT = u'fakesink'

#: The log format used for debug logging.
#:
#: See http://docs.python.org/library/logging.html#formatter-objects for
#: details on the format.
DEBUG_LOG_FORMAT = u'%(levelname)-8s %(asctime)s' + \
    ' [%(process)d:%(threadName)s] %(name)s\n  %(message)s'

#: The file to dump debug log data to when Mopidy is run with the
#: :option:`--save-debug-log` option.
#:
#: Default::
#:
#:     DEBUG_LOG_FILENAME = u'mopidy.log'
DEBUG_LOG_FILENAME = u'mopidy.log'

#: Location of the Mopidy .desktop file.
#:
#: Used by :mod:`mopidy.frontends.mpris`.
#:
#: Default::
#:
#:     DESKTOP_FILE = u'/usr/share/applications/mopidy.desktop'
DESKTOP_FILE = u'/usr/share/applications/mopidy.desktop'

#: List of server frontends to use.
#:
#: Default::
#:
#:     FRONTENDS = (
#:         u'mopidy.frontends.mpd.MpdFrontend',
#:         u'mopidy.frontends.lastfm.LastfmFrontend',
#:         u'mopidy.frontends.mpris.MprisFrontend',
#:     )
FRONTENDS = (
    u'mopidy.frontends.mpd.MpdFrontend',
    u'mopidy.frontends.lastfm.LastfmFrontend',
    u'mopidy.frontends.mpris.MprisFrontend',
)

#: Your `Last.fm <http://www.last.fm/>`_ username.
#:
#: Used by :mod:`mopidy.frontends.lastfm`.
LASTFM_USERNAME = u''

#: Your `Last.fm <http://www.last.fm/>`_ password.
#:
#: Used by :mod:`mopidy.frontends.lastfm`.
LASTFM_PASSWORD = u''

#: Path to folder with local music.
#:
#: Used by :mod:`mopidy.backends.local`.
#:
#: Default::
#:
#:    # Defaults to asking glib where music is stored, fallback is ~/music
#:    LOCAL_MUSIC_PATH = None
LOCAL_MUSIC_PATH = None

#: Path to playlist folder with m3u files for local music.
#:
#: Used by :mod:`mopidy.backends.local`.
#:
#: Default::
#:
#:    LOCAL_PLAYLIST_PATH = None # Implies $XDG_DATA_DIR/mopidy/playlists
LOCAL_PLAYLIST_PATH = None

#: Path to tag cache for local music.
#:
#: Used by :mod:`mopidy.backends.local`.
#:
#: Default::
#:
#:    LOCAL_TAG_CACHE_FILE = None # Implies $XDG_DATA_DIR/mopidy/tag_cache
LOCAL_TAG_CACHE_FILE = None

#: Sound mixer to use. See :mod:`mopidy.mixers` for all available mixers.
#:
#: Default::
#:
#:     MIXER = u'mopidy.mixers.gstreamer_software.GStreamerSoftwareMixer'
MIXER = u'mopidy.mixers.gstreamer_software.GStreamerSoftwareMixer'

#: ALSA mixer only. What mixer control to use. If set to :class:`False`, first
#: ``Master`` and then ``PCM`` will be tried.
#:
#: Example: ``Master Front``. Default: :class:`False`
MIXER_ALSA_CONTROL = False

#: External mixers only. Which port the mixer is connected to.
#:
#: This must point to the device port like ``/dev/ttyUSB0``.
#:
#: Default: :class:`None`
MIXER_EXT_PORT = None

#: External mixers only. What input source the external mixer should use.
#:
#: Example: ``Aux``. Default: :class:`None`
MIXER_EXT_SOURCE = None

#: External mixers only. What state Speakers A should be in.
#:
#: Default: :class:`None`.
MIXER_EXT_SPEAKERS_A = None

#: External mixers only. What state Speakers B should be in.
#:
#: Default: :class:`None`.
MIXER_EXT_SPEAKERS_B = None

#: The maximum volume. Integer in the range 0 to 100.
#:
#: If this settings is set to 80, the mixer will set the actual volume to 80
#: when asked to set it to 100.
#:
#: Default::
#:
#:     MIXER_MAX_VOLUME = 100
MIXER_MAX_VOLUME = 100

#: Which address Mopidy's MPD server should bind to.
#:
#:Examples:
#:
#: ``127.0.0.1``
#:     Listens only on the IPv4 loopback interface. Default.
#: ``::1``
#:     Listens only on the IPv6 loopback interface.
#: ``0.0.0.0``
#:     Listens on all IPv4 interfaces.
#: ``::``
#:     Listens on all interfaces, both IPv4 and IPv6.
MPD_SERVER_HOSTNAME = u'127.0.0.1'

#: Which TCP port Mopidy's MPD server should listen to.
#:
#: Default: 6600
MPD_SERVER_PORT = 6600

#: The password required for connecting to the MPD server.
#:
#: Default: :class:`None`, which means no password required.
MPD_SERVER_PASSWORD = None

#: The maximum number of concurrent connections the MPD server will accept.
#:
#: Default: 20
MPD_SERVER_MAX_CONNECTIONS = 20

#: List of outputs to use. See :mod:`mopidy.outputs` for all available
#: backends
#:
#: Default::
#:
#:     OUTPUTS = (
#:         u'mopidy.outputs.local.LocalOutput',
#:     )
OUTPUTS = (
    u'mopidy.outputs.local.LocalOutput',
)

#: Hostname of the SHOUTcast server which Mopidy should stream audio to.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:    SHOUTCAST_OUTPUT_HOSTNAME = u'127.0.0.1'
SHOUTCAST_OUTPUT_HOSTNAME = u'127.0.0.1'

#: Port of the SHOUTcast server.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:    SHOUTCAST_OUTPUT_PORT = 8000
SHOUTCAST_OUTPUT_PORT = 8000

#: User to authenticate as against SHOUTcast server.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:    SHOUTCAST_OUTPUT_USERNAME = u'source'
SHOUTCAST_OUTPUT_USERNAME = u'source'

#: Password to authenticate with against SHOUTcast server.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:    SHOUTCAST_OUTPUT_PASSWORD = u'hackme'
SHOUTCAST_OUTPUT_PASSWORD = u'hackme'

#: Mountpoint to use for the stream on the SHOUTcast server.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:    SHOUTCAST_OUTPUT_MOUNT = u'/stream'
SHOUTCAST_OUTPUT_MOUNT = u'/stream'

#: Encoder to use to process audio data before streaming to SHOUTcast server.
#:
#: Used by :mod:`mopidy.outputs.shoutcast`.
#:
#: Default::
#:
#:     SHOUTCAST_OUTPUT_ENCODER = u'lame mode=stereo bitrate=320'
SHOUTCAST_OUTPUT_ENCODER = u'lame mode=stereo bitrate=320'

#: Path to the Spotify cache.
#:
#: Used by :mod:`mopidy.backends.spotify`.
SPOTIFY_CACHE_PATH = None

#: Your Spotify Premium username.
#:
#: Used by :mod:`mopidy.backends.spotify`.
SPOTIFY_USERNAME = u''

#: Your Spotify Premium password.
#:
#: Used by :mod:`mopidy.backends.spotify`.
SPOTIFY_PASSWORD = u''

#: Spotify preferred bitrate.
#:
#: Available values are 96, 160, and 320.
#:
#: Used by :mod:`mopidy.backends.spotify`.
#
#: Default::
#:
#:     SPOTIFY_BITRATE = 160
SPOTIFY_BITRATE = 160



from distutils.version import StrictVersion as SV
import platform

from mopidy import get_plain_version, get_platform, get_python

from tests import unittest


class VersionTest(unittest.TestCase):
    def test_current_version_is_parsable_as_a_strict_version_number(self):
        SV(get_plain_version())

    def test_versions_can_be_strictly_ordered(self):
        self.assert_(SV('0.1.0a0') < SV('0.1.0a1'))
        self.assert_(SV('0.1.0a1') < SV('0.1.0a2'))
        self.assert_(SV('0.1.0a2') < SV('0.1.0a3'))
        self.assert_(SV('0.1.0a3') < SV('0.1.0'))
        self.assert_(SV('0.1.0') < SV('0.2.0'))
        self.assert_(SV('0.1.0') < SV('1.0.0'))
        self.assert_(SV('0.2.0') < SV('0.3.0'))
        self.assert_(SV('0.3.0') < SV('0.3.1'))
        self.assert_(SV('0.3.1') < SV('0.4.0'))
        self.assert_(SV('0.4.0') < SV('0.4.1'))
        self.assert_(SV('0.4.1') < SV('0.5.0'))
        self.assert_(SV('0.5.0') < SV('0.6.0'))
        self.assert_(SV('0.6.0') < SV(get_plain_version()))
        self.assert_(SV(get_plain_version()) < SV('0.7.0'))

    def test_get_platform_contains_platform(self):
        self.assert_(platform.platform() in get_platform())

    def test_get_python_contains_python_implementation(self):
        self.assert_(platform.python_implementation() in get_python())

    def test_get_python_contains_python_version(self):
        self.assert_(platform.python_version() in get_python())



from mopidy.backends import base as backend
from mopidy.models import Track

from tests import unittest
from tests.frontends.mpd import protocol

PAUSED = backend.PlaybackController.PAUSED
PLAYING = backend.PlaybackController.PLAYING
STOPPED = backend.PlaybackController.STOPPED


class PlaybackOptionsHandlerTest(protocol.BaseTestCase):
    def test_consume_off(self):
        self.sendRequest(u'consume "0"')
        self.assertFalse(self.backend.playback.consume.get())
        self.assertInResponse(u'OK')

    def test_consume_off_without_quotes(self):
        self.sendRequest(u'consume 0')
        self.assertFalse(self.backend.playback.consume.get())
        self.assertInResponse(u'OK')

    def test_consume_on(self):
        self.sendRequest(u'consume "1"')
        self.assertTrue(self.backend.playback.consume.get())
        self.assertInResponse(u'OK')

    def test_consume_on_without_quotes(self):
        self.sendRequest(u'consume 1')
        self.assertTrue(self.backend.playback.consume.get())
        self.assertInResponse(u'OK')

    def test_crossfade(self):
        self.sendRequest(u'crossfade "10"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_random_off(self):
        self.sendRequest(u'random "0"')
        self.assertFalse(self.backend.playback.random.get())
        self.assertInResponse(u'OK')

    def test_random_off_without_quotes(self):
        self.sendRequest(u'random 0')
        self.assertFalse(self.backend.playback.random.get())
        self.assertInResponse(u'OK')

    def test_random_on(self):
        self.sendRequest(u'random "1"')
        self.assertTrue(self.backend.playback.random.get())
        self.assertInResponse(u'OK')

    def test_random_on_without_quotes(self):
        self.sendRequest(u'random 1')
        self.assertTrue(self.backend.playback.random.get())
        self.assertInResponse(u'OK')

    def test_repeat_off(self):
        self.sendRequest(u'repeat "0"')
        self.assertFalse(self.backend.playback.repeat.get())
        self.assertInResponse(u'OK')

    def test_repeat_off_without_quotes(self):
        self.sendRequest(u'repeat 0')
        self.assertFalse(self.backend.playback.repeat.get())
        self.assertInResponse(u'OK')

    def test_repeat_on(self):
        self.sendRequest(u'repeat "1"')
        self.assertTrue(self.backend.playback.repeat.get())
        self.assertInResponse(u'OK')

    def test_repeat_on_without_quotes(self):
        self.sendRequest(u'repeat 1')
        self.assertTrue(self.backend.playback.repeat.get())
        self.assertInResponse(u'OK')

    def test_setvol_below_min(self):
        self.sendRequest(u'setvol "-10"')
        self.assertEqual(0, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_min(self):
        self.sendRequest(u'setvol "0"')
        self.assertEqual(0, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_middle(self):
        self.sendRequest(u'setvol "50"')
        self.assertEqual(50, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_max(self):
        self.sendRequest(u'setvol "100"')
        self.assertEqual(100, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_above_max(self):
        self.sendRequest(u'setvol "110"')
        self.assertEqual(100, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_plus_is_ignored(self):
        self.sendRequest(u'setvol "+10"')
        self.assertEqual(10, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_setvol_without_quotes(self):
        self.sendRequest(u'setvol 50')
        self.assertEqual(50, self.mixer.volume.get())
        self.assertInResponse(u'OK')

    def test_single_off(self):
        self.sendRequest(u'single "0"')
        self.assertFalse(self.backend.playback.single.get())
        self.assertInResponse(u'OK')

    def test_single_off_without_quotes(self):
        self.sendRequest(u'single 0')
        self.assertFalse(self.backend.playback.single.get())
        self.assertInResponse(u'OK')

    def test_single_on(self):
        self.sendRequest(u'single "1"')
        self.assertTrue(self.backend.playback.single.get())
        self.assertInResponse(u'OK')

    def test_single_on_without_quotes(self):
        self.sendRequest(u'single 1')
        self.assertTrue(self.backend.playback.single.get())
        self.assertInResponse(u'OK')

    def test_replay_gain_mode_off(self):
        self.sendRequest(u'replay_gain_mode "off"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_replay_gain_mode_track(self):
        self.sendRequest(u'replay_gain_mode "track"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_replay_gain_mode_album(self):
        self.sendRequest(u'replay_gain_mode "album"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_replay_gain_status_default(self):
        self.sendRequest(u'replay_gain_status')
        self.assertInResponse(u'OK')
        self.assertInResponse(u'off')

    @unittest.SkipTest
    def test_replay_gain_status_off(self):
        pass

    @unittest.SkipTest
    def test_replay_gain_status_track(self):
        pass

    @unittest.SkipTest
    def test_replay_gain_status_album(self):
        pass


class PlaybackControlHandlerTest(protocol.BaseTestCase):
    def test_next(self):
        self.sendRequest(u'next')
        self.assertInResponse(u'OK')

    def test_pause_off(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'play "0"')
        self.sendRequest(u'pause "1"')
        self.sendRequest(u'pause "0"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_pause_on(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'play "0"')
        self.sendRequest(u'pause "1"')
        self.assertEqual(PAUSED, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_pause_toggle(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'play "0"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

        self.sendRequest(u'pause')
        self.assertEqual(PAUSED, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

        self.sendRequest(u'pause')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_play_without_pos(self):
        self.backend.current_playlist.append([Track()])
        self.backend.playback.state = PAUSED

        self.sendRequest(u'play')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_play_with_pos(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'play "0"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_play_with_pos_without_quotes(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'play 0')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_play_with_pos_out_of_bounds(self):
        self.backend.current_playlist.append([])

        self.sendRequest(u'play "0"')
        self.assertEqual(STOPPED, self.backend.playback.state.get())
        self.assertInResponse(u'ACK [2@0] {play} Bad song index')

    def test_play_minus_one_plays_first_in_playlist_if_no_current_track(self):
        self.assertEqual(self.backend.playback.current_track.get(), None)
        self.backend.current_playlist.append([Track(uri='a'), Track(uri='b')])

        self.sendRequest(u'play "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertEqual('a', self.backend.playback.current_track.get().uri)
        self.assertInResponse(u'OK')

    def test_play_minus_one_plays_current_track_if_current_track_is_set(self):
        self.backend.current_playlist.append([Track(uri='a'), Track(uri='b')])
        self.assertEqual(self.backend.playback.current_track.get(), None)
        self.backend.playback.play()
        self.backend.playback.next()
        self.backend.playback.stop()
        self.assertNotEqual(self.backend.playback.current_track.get(), None)

        self.sendRequest(u'play "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertEqual('b', self.backend.playback.current_track.get().uri)
        self.assertInResponse(u'OK')

    def test_play_minus_one_on_empty_playlist_does_not_ack(self):
        self.backend.current_playlist.clear()

        self.sendRequest(u'play "-1"')
        self.assertEqual(STOPPED, self.backend.playback.state.get())
        self.assertEqual(None, self.backend.playback.current_track.get())
        self.assertInResponse(u'OK')

    def test_play_minus_is_ignored_if_playing(self):
        self.backend.current_playlist.append([Track(length=40000)])
        self.backend.playback.seek(30000)
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertEquals(PLAYING, self.backend.playback.state.get())

        self.sendRequest(u'play "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_play_minus_one_resumes_if_paused(self):
        self.backend.current_playlist.append([Track(length=40000)])
        self.backend.playback.seek(30000)
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertEquals(PLAYING, self.backend.playback.state.get())
        self.backend.playback.pause()
        self.assertEquals(PAUSED, self.backend.playback.state.get())

        self.sendRequest(u'play "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_playid(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'playid "0"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertInResponse(u'OK')

    def test_playid_minus_one_plays_first_in_playlist_if_no_current_track(self):
        self.assertEqual(self.backend.playback.current_track.get(), None)
        self.backend.current_playlist.append([Track(uri='a'), Track(uri='b')])

        self.sendRequest(u'playid "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertEqual('a', self.backend.playback.current_track.get().uri)
        self.assertInResponse(u'OK')

    def test_playid_minus_one_plays_current_track_if_current_track_is_set(self):
        self.backend.current_playlist.append([Track(uri='a'), Track(uri='b')])
        self.assertEqual(self.backend.playback.current_track.get(), None)
        self.backend.playback.play()
        self.backend.playback.next()
        self.backend.playback.stop()
        self.assertNotEqual(None, self.backend.playback.current_track.get())

        self.sendRequest(u'playid "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assertEqual('b', self.backend.playback.current_track.get().uri)
        self.assertInResponse(u'OK')

    def test_playid_minus_one_on_empty_playlist_does_not_ack(self):
        self.backend.current_playlist.clear()

        self.sendRequest(u'playid "-1"')
        self.assertEqual(STOPPED, self.backend.playback.state.get())
        self.assertEqual(None, self.backend.playback.current_track.get())
        self.assertInResponse(u'OK')

    def test_playid_minus_is_ignored_if_playing(self):
        self.backend.current_playlist.append([Track(length=40000)])
        self.backend.playback.seek(30000)
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertEquals(PLAYING, self.backend.playback.state.get())

        self.sendRequest(u'playid "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_playid_minus_one_resumes_if_paused(self):
        self.backend.current_playlist.append([Track(length=40000)])
        self.backend.playback.seek(30000)
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertEquals(PLAYING, self.backend.playback.state.get())
        self.backend.playback.pause()
        self.assertEquals(PAUSED, self.backend.playback.state.get())

        self.sendRequest(u'playid "-1"')
        self.assertEqual(PLAYING, self.backend.playback.state.get())
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_playid_which_does_not_exist(self):
        self.backend.current_playlist.append([Track()])

        self.sendRequest(u'playid "12345"')
        self.assertInResponse(u'ACK [50@0] {playid} No such song')

    def test_previous(self):
        self.sendRequest(u'previous')
        self.assertInResponse(u'OK')

    def test_seek(self):
        self.backend.current_playlist.append([Track(length=40000)])

        self.sendRequest(u'seek "0"')
        self.sendRequest(u'seek "0" "30"')
        self.assert_(self.backend.playback.time_position >= 30000)
        self.assertInResponse(u'OK')

    def test_seek_with_songpos(self):
        seek_track = Track(uri='2', length=40000)
        self.backend.current_playlist.append(
            [Track(uri='1', length=40000), seek_track])

        self.sendRequest(u'seek "1" "30"')
        self.assertEqual(self.backend.playback.current_track.get(), seek_track)
        self.assertInResponse(u'OK')

    def test_seek_without_quotes(self):
        self.backend.current_playlist.append([Track(length=40000)])

        self.sendRequest(u'seek 0')
        self.sendRequest(u'seek 0 30')
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_seekid(self):
        self.backend.current_playlist.append([Track(length=40000)])
        self.sendRequest(u'seekid "0" "30"')
        self.assert_(self.backend.playback.time_position.get() >= 30000)
        self.assertInResponse(u'OK')

    def test_seekid_with_cpid(self):
        seek_track = Track(uri='2', length=40000)
        self.backend.current_playlist.append(
            [Track(length=40000), seek_track])

        self.sendRequest(u'seekid "1" "30"')
        self.assertEqual(1, self.backend.playback.current_cpid.get())
        self.assertEqual(seek_track, self.backend.playback.current_track.get())
        self.assertInResponse(u'OK')

    def test_stop(self):
        self.sendRequest(u'stop')
        self.assertEqual(STOPPED, self.backend.playback.state.get())
        self.assertInResponse(u'OK')



from mopidy.listeners import BackendListener
from mopidy.models import Track

from tests import unittest


class BackendListenerTest(unittest.TestCase):
    def setUp(self):
        self.listener = BackendListener()

    def test_listener_has_default_impl_for_track_playback_paused(self):
        self.listener.track_playback_paused(Track(), 0)

    def test_listener_has_default_impl_for_track_playback_resumed(self):
        self.listener.track_playback_resumed(Track(), 0)

    def test_listener_has_default_impl_for_track_playback_started(self):
        self.listener.track_playback_started(Track())

    def test_listener_has_default_impl_for_track_playback_ended(self):
        self.listener.track_playback_ended(Track(), 0)

    def test_listener_has_default_impl_for_playback_state_changed(self):
        self.listener.playback_state_changed()

    def test_listener_has_default_impl_for_playlist_changed(self):
        self.listener.playlist_changed()

    def test_listener_has_default_impl_for_options_changed(self):
        self.listener.options_changed()

    def test_listener_has_default_impl_for_volume_changed(self):
        self.listener.volume_changed()

    def test_listener_has_default_impl_for_seeked(self):
        self.listener.seeked()



# encoding: utf-8

import os
import tempfile

from mopidy.utils.path import path_to_uri
from mopidy.backends.local.translator import parse_m3u, parse_mpd_tag_cache
from mopidy.models import Track, Artist, Album

from tests import unittest, path_to_data_dir

song1_path = path_to_data_dir('song1.mp3')
song2_path = path_to_data_dir('song2.mp3')
encoded_path = path_to_data_dir(u'æøå.mp3')
song1_uri = path_to_uri(song1_path)
song2_uri = path_to_uri(song2_path)
encoded_uri = path_to_uri(encoded_path)

# FIXME use mock instead of tempfile.NamedTemporaryFile


class M3UToUriTest(unittest.TestCase):
    def test_empty_file(self):
        uris = parse_m3u(path_to_data_dir('empty.m3u'))
        self.assertEqual([], uris)

    def test_basic_file(self):
        uris = parse_m3u(path_to_data_dir('one.m3u'))
        self.assertEqual([song1_uri], uris)

    def test_file_with_comment(self):
        uris = parse_m3u(path_to_data_dir('comment.m3u'))
        self.assertEqual([song1_uri], uris)

    def test_file_with_absolute_files(self):
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmp.write(song1_path)
        try:
            uris = parse_m3u(tmp.name)
            self.assertEqual([song1_uri], uris)
        finally:
            if os.path.exists(tmp.name):
                os.remove(tmp.name)

    def test_file_with_multiple_absolute_files(self):
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmp.write(song1_path+'\n')
            tmp.write('# comment \n')
            tmp.write(song2_path)
        try:
            uris = parse_m3u(tmp.name)
            self.assertEqual([song1_uri, song2_uri], uris)
        finally:
            if os.path.exists(tmp.name):
                os.remove(tmp.name)


    def test_file_with_uri(self):
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmp.write(song1_uri)
        try:
            uris = parse_m3u(tmp.name)
            self.assertEqual([song1_uri], uris)
        finally:
            if os.path.exists(tmp.name):
                os.remove(tmp.name)

    def test_encoding_is_latin1(self):
        uris = parse_m3u(path_to_data_dir('encoding.m3u'))
        self.assertEqual([encoded_uri], uris)

    def test_open_missing_file(self):
        uris = parse_m3u(path_to_data_dir('non-existant.m3u'))
        self.assertEqual([], uris)


class URItoM3UTest(unittest.TestCase):
    pass

expected_artists = [Artist(name='name')]
expected_albums = [Album(name='albumname', artists=expected_artists,
    num_tracks=2)]
expected_tracks = []

def generate_track(path, ident):
    uri = path_to_uri(path_to_data_dir(path))
    track = Track(name='trackname', artists=expected_artists, track_no=1,
        album=expected_albums[0], length=4000, uri=uri)
    expected_tracks.append(track)

generate_track('song1.mp3', 6)
generate_track('song2.mp3', 7)
generate_track('song3.mp3', 8)
generate_track('subdir1/song4.mp3', 2)
generate_track('subdir1/song5.mp3', 3)
generate_track('subdir2/song6.mp3', 4)
generate_track('subdir2/song7.mp3', 5)
generate_track('subdir1/subsubdir/song8.mp3', 0)
generate_track('subdir1/subsubdir/song9.mp3', 1)

class MPDTagCacheToTracksTest(unittest.TestCase):
    def test_emtpy_cache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('empty_tag_cache'),
            path_to_data_dir(''))
        self.assertEqual(set(), tracks)

    def test_simple_cache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('simple_tag_cache'),
            path_to_data_dir(''))
        uri = path_to_uri(path_to_data_dir('song1.mp3'))
        track = Track(name='trackname', artists=expected_artists, track_no=1,
            album=expected_albums[0], length=4000, uri=uri)
        self.assertEqual(set([track]), tracks)

    def test_advanced_cache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('advanced_tag_cache'),
             path_to_data_dir(''))
        self.assertEqual(set(expected_tracks), tracks)

    def test_unicode_cache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('utf8_tag_cache'),
             path_to_data_dir(''))

        uri = path_to_uri(path_to_data_dir('song1.mp3'))
        artists = [Artist(name=u'æøå')]
        album = Album(name=u'æøå', artists=artists)
        track = Track(uri=uri, name=u'æøå', artists=artists,
            album=album, length=4000)

        self.assertEqual(track, list(tracks)[0])

    @unittest.SkipTest
    def test_misencoded_cache(self):
        # FIXME not sure if this can happen
        pass

    def test_cache_with_blank_track_info(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('blank_tag_cache'),
            path_to_data_dir(''))
        uri = path_to_uri(path_to_data_dir('song1.mp3'))
        self.assertEqual(set([Track(uri=uri, length=4000)]), tracks)

    def test_musicbrainz_tagcache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('musicbrainz_tag_cache'),
            path_to_data_dir(''))
        artist = list(expected_tracks[0].artists)[0].copy(
            musicbrainz_id='7364dea6-ca9a-48e3-be01-b44ad0d19897')
        albumartist = list(expected_tracks[0].artists)[0].copy(
            name='albumartistname',
            musicbrainz_id='7364dea6-ca9a-48e3-be01-b44ad0d19897')
        album = expected_tracks[0].album.copy(artists=[albumartist],
            musicbrainz_id='cb5f1603-d314-4c9c-91e5-e295cfb125d2')
        track = expected_tracks[0].copy(artists=[artist], album=album,
            musicbrainz_id='90488461-8c1f-4a4e-826b-4c6dc70801f0')

        self.assertEqual(track, list(tracks)[0])

    def test_albumartist_tag_cache(self):
        tracks = parse_mpd_tag_cache(path_to_data_dir('albumartist_tag_cache'),
            path_to_data_dir(''))
        uri = path_to_uri(path_to_data_dir('song1.mp3'))
        artist = Artist(name='albumartistname')
        album = expected_albums[0].copy(artists=[artist])
        track = Track(name='trackname', artists=expected_artists, track_no=1,
            album=album, length=4000, uri=uri)
        self.assertEqual(track, list(tracks)[0])



import mock

from mopidy import settings
from mopidy.backends.dummy import DummyBackend
from mopidy.frontends.mpris import objects

from tests import unittest


class RootInterfaceTest(unittest.TestCase):
    def setUp(self):
        objects.exit_process = mock.Mock()
        objects.MprisObject._connect_to_dbus = mock.Mock()
        self.backend = DummyBackend.start().proxy()
        self.mpris = objects.MprisObject()

    def tearDown(self):
        self.backend.stop()

    def test_constructor_connects_to_dbus(self):
        self.assert_(self.mpris._connect_to_dbus.called)

    def test_can_raise_returns_false(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'CanRaise')
        self.assertFalse(result)

    def test_raise_does_nothing(self):
        self.mpris.Raise()

    def test_can_quit_returns_true(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'CanQuit')
        self.assertTrue(result)

    def test_quit_should_stop_all_actors(self):
        self.mpris.Quit()
        self.assert_(objects.exit_process.called)

    def test_has_track_list_returns_false(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'HasTrackList')
        self.assertFalse(result)

    def test_identify_is_mopidy(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'Identity')
        self.assertEquals(result, 'Mopidy')

    def test_desktop_entry_is_mopidy(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'DesktopEntry')
        self.assertEquals(result, 'mopidy')

    def test_desktop_entry_is_based_on_DESKTOP_FILE_setting(self):
        settings.runtime['DESKTOP_FILE'] = '/tmp/foo.desktop'
        result = self.mpris.Get(objects.ROOT_IFACE, 'DesktopEntry')
        self.assertEquals(result, 'foo')
        settings.runtime.clear()

    def test_supported_uri_schemes_is_empty(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'SupportedUriSchemes')
        self.assertEquals(len(result), 1)
        self.assertEquals(result[0], 'dummy')

    def test_supported_mime_types_is_empty(self):
        result = self.mpris.Get(objects.ROOT_IFACE, 'SupportedMimeTypes')
        self.assertEquals(len(result), 0)



import os
import re

from mopidy import settings
from mopidy.utils.path import mtime as get_mtime
from mopidy.frontends.mpd import protocol
from mopidy.utils.path import uri_to_path, split_path

def track_to_mpd_format(track, position=None, cpid=None):
    """
    Format track for output to MPD client.

    :param track: the track
    :type track: :class:`mopidy.models.Track`
    :param position: track's position in playlist
    :type position: integer
    :param cpid: track's CPID (current playlist ID)
    :type cpid: integer
    :param key: if we should set key
    :type key: boolean
    :param mtime: if we should set mtime
    :type mtime: boolean
    :rtype: list of two-tuples
    """
    result = [
        ('file', track.uri or ''),
        ('Time', track.length and (track.length // 1000) or 0),
        ('Artist', artists_to_mpd_format(track.artists)),
        ('Title', track.name or ''),
        ('Album', track.album and track.album.name or ''),
        ('Date', track.date or ''),
    ]
    if track.album is not None and track.album.num_tracks != 0:
        result.append(('Track', '%d/%d' % (
            track.track_no, track.album.num_tracks)))
    else:
        result.append(('Track', track.track_no))
    if track.album is not None and track.album.artists:
        artists = artists_to_mpd_format(track.album.artists)
        result.append(('AlbumArtist', artists))
    if position is not None and cpid is not None:
        result.append(('Pos', position))
        result.append(('Id', cpid))
    if track.album is not None and track.album.musicbrainz_id is not None:
        result.append(('MUSICBRAINZ_ALBUMID', track.album.musicbrainz_id))
    # FIXME don't use first and best artist?
    # FIXME don't duplicate following code?
    if track.album is not None and track.album.artists:
        artists = filter(lambda a: a.musicbrainz_id is not None,
            track.album.artists)
        if artists:
            result.append(
                ('MUSICBRAINZ_ALBUMARTISTID', artists[0].musicbrainz_id))
    if track.artists:
        artists = filter(lambda a: a.musicbrainz_id is not None, track.artists)
        if artists:
            result.append(('MUSICBRAINZ_ARTISTID', artists[0].musicbrainz_id))
    if track.musicbrainz_id is not None:
        result.append(('MUSICBRAINZ_TRACKID', track.musicbrainz_id))
    return result

MPD_KEY_ORDER = '''
    key file Time Artist AlbumArtist Title Album Track Date MUSICBRAINZ_ALBUMID
    MUSICBRAINZ_ALBUMARTISTID MUSICBRAINZ_ARTISTID MUSICBRAINZ_TRACKID mtime
'''.split()

def order_mpd_track_info(result):
    """
    Order results from :func:`mopidy.frontends.mpd.translator.track_to_mpd_format`
    so that it matches MPD's ordering. Simply a cosmetic fix for easier
    diffing of tag_caches.

    :param result: the track info
    :type result: list of tuples
    :rtype: list of tuples
    """
    return sorted(result, key=lambda i: MPD_KEY_ORDER.index(i[0]))

def artists_to_mpd_format(artists):
    """
    Format track artists for output to MPD client.

    :param artists: the artists
    :type track: array of :class:`mopidy.models.Artist`
    :rtype: string
    """
    artists = list(artists)
    artists.sort(key=lambda a: a.name)
    return u', '.join([a.name for a in artists if a.name])

def tracks_to_mpd_format(tracks, start=0, end=None, cpids=None):
    """
    Format list of tracks for output to MPD client.

    Optionally limit output to the slice ``[start:end]`` of the list.

    :param tracks: the tracks
    :type tracks: list of :class:`mopidy.models.Track`
    :param start: position of first track to include in output
    :type start: int (positive or negative)
    :param end: position after last track to include in output
    :type end: int (positive or negative) or :class:`None` for end of list
    :rtype: list of lists of two-tuples
    """
    if end is None:
        end = len(tracks)
    tracks = tracks[start:end]
    positions = range(start, end)
    cpids = cpids and cpids[start:end] or [None for _ in tracks]
    assert len(tracks) == len(positions) == len(cpids)
    result = []
    for track, position, cpid in zip(tracks, positions, cpids):
        result.append(track_to_mpd_format(track, position, cpid))
    return result

def playlist_to_mpd_format(playlist, *args, **kwargs):
    """
    Format playlist for output to MPD client.

    Arguments as for :func:`tracks_to_mpd_format`, except the first one.
    """
    return tracks_to_mpd_format(playlist.tracks, *args, **kwargs)

def tracks_to_tag_cache_format(tracks):
    """
    Format list of tracks for output to MPD tag cache

    :param tracks: the tracks
    :type tracks: list of :class:`mopidy.models.Track`
    :rtype: list of lists of two-tuples
    """
    result = [
        ('info_begin',),
        ('mpd_version', protocol.VERSION),
        ('fs_charset', protocol.ENCODING),
        ('info_end',)
    ]
    tracks.sort(key=lambda t: t.uri)
    _add_to_tag_cache(result, *tracks_to_directory_tree(tracks))
    return result

def _add_to_tag_cache(result, folders, files):
    music_folder = settings.LOCAL_MUSIC_PATH
    regexp = '^' + re.escape(music_folder).rstrip('/') + '/?'

    for path, entry in folders.items():
        name = os.path.split(path)[1]
        mtime = get_mtime(os.path.join(music_folder, path))
        result.append(('directory', path))
        result.append(('mtime', mtime))
        result.append(('begin', name))
        _add_to_tag_cache(result, *entry)
        result.append(('end', name))

    result.append(('songList begin',))
    for track in files:
        track_result = dict(track_to_mpd_format(track))
        path = uri_to_path(track_result['file'])
        track_result['mtime'] = get_mtime(path)
        track_result['file'] = re.sub(regexp, '', path)
        track_result['key'] = os.path.basename(track_result['file'])
        track_result = order_mpd_track_info(track_result.items())
        result.extend(track_result)
    result.append(('songList end',))

def tracks_to_directory_tree(tracks):
    directories = ({}, [])
    for track in tracks:
        path = u''
        current = directories

        local_folder = settings.LOCAL_MUSIC_PATH
        track_path = uri_to_path(track.uri)
        track_path = re.sub('^' + re.escape(local_folder), '', track_path)
        track_dir = os.path.dirname(track_path)

        for part in split_path(track_dir):
            path = os.path.join(path, part)
            if path not in current[0]:
                current[0][path] = ({}, [])
            current = current[0][path]
        current[1].append(track)
    return directories



from mopidy.models import Track

from tests.frontends.mpd import protocol


class CurrentPlaylistHandlerTest(protocol.BaseTestCase):
    def test_add(self):
        needle = Track(uri='dummy://foo')
        self.backend.library.provider.dummy_library = [
            Track(), Track(), needle, Track()]
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'add "dummy://foo"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 6)
        self.assertEqual(self.backend.current_playlist.tracks.get()[5], needle)
        self.assertEqualResponse(u'OK')

    def test_add_with_uri_not_found_in_library_should_ack(self):
        self.sendRequest(u'add "dummy://foo"')
        self.assertEqualResponse(
            u'ACK [50@0] {add} directory or file not found')

    def test_add_with_empty_uri_should_add_all_known_tracks_and_ok(self):
        self.sendRequest(u'add ""')
        # TODO check that we add all tracks (we currently don't)
        self.assertInResponse(u'OK')

    def test_addid_without_songpos(self):
        needle = Track(uri='dummy://foo')
        self.backend.library.provider.dummy_library = [
            Track(), Track(), needle, Track()]
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'addid "dummy://foo"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 6)
        self.assertEqual(self.backend.current_playlist.tracks.get()[5], needle)
        self.assertInResponse(u'Id: %d' %
            self.backend.current_playlist.cp_tracks.get()[5][0])
        self.assertInResponse(u'OK')

    def test_addid_with_empty_uri_acks(self):
        self.sendRequest(u'addid ""')
        self.assertEqualResponse(u'ACK [50@0] {addid} No such song')

    def test_addid_with_songpos(self):
        needle = Track(uri='dummy://foo')
        self.backend.library.provider.dummy_library = [
            Track(), Track(), needle, Track()]
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'addid "dummy://foo" "3"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 6)
        self.assertEqual(self.backend.current_playlist.tracks.get()[3], needle)
        self.assertInResponse(u'Id: %d' %
            self.backend.current_playlist.cp_tracks.get()[3][0])
        self.assertInResponse(u'OK')

    def test_addid_with_songpos_out_of_bounds_should_ack(self):
        needle = Track(uri='dummy://foo')
        self.backend.library.provider.dummy_library = [
            Track(), Track(), needle, Track()]
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'addid "dummy://foo" "6"')
        self.assertEqualResponse(u'ACK [2@0] {addid} Bad song index')

    def test_addid_with_uri_not_found_in_library_should_ack(self):
        self.sendRequest(u'addid "dummy://foo"')
        self.assertEqualResponse(u'ACK [50@0] {addid} No such song')

    def test_clear(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'clear')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 0)
        self.assertEqual(self.backend.playback.current_track.get(), None)
        self.assertInResponse(u'OK')

    def test_delete_songpos(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'delete "%d"' %
            self.backend.current_playlist.cp_tracks.get()[2][0])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 4)
        self.assertInResponse(u'OK')

    def test_delete_songpos_out_of_bounds(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'delete "5"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)
        self.assertEqualResponse(u'ACK [2@0] {delete} Bad song index')

    def test_delete_open_range(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'delete "1:"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 1)
        self.assertInResponse(u'OK')

    def test_delete_closed_range(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'delete "1:3"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 3)
        self.assertInResponse(u'OK')

    def test_delete_range_out_of_bounds(self):
        self.backend.current_playlist.append(
            [Track(), Track(), Track(), Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)

        self.sendRequest(u'delete "5:7"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 5)
        self.assertEqualResponse(u'ACK [2@0] {delete} Bad song index')

    def test_deleteid(self):
        self.backend.current_playlist.append([Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 2)

        self.sendRequest(u'deleteid "1"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 1)
        self.assertInResponse(u'OK')

    def test_deleteid_does_not_exist(self):
        self.backend.current_playlist.append([Track(), Track()])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 2)

        self.sendRequest(u'deleteid "12345"')
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 2)
        self.assertEqualResponse(u'ACK [50@0] {deleteid} No such song')

    def test_move_songpos(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'move "1" "0"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'b')
        self.assertEqual(tracks[1].name, 'a')
        self.assertEqual(tracks[2].name, 'c')
        self.assertEqual(tracks[3].name, 'd')
        self.assertEqual(tracks[4].name, 'e')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')

    def test_move_open_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'move "2:" "0"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'c')
        self.assertEqual(tracks[1].name, 'd')
        self.assertEqual(tracks[2].name, 'e')
        self.assertEqual(tracks[3].name, 'f')
        self.assertEqual(tracks[4].name, 'a')
        self.assertEqual(tracks[5].name, 'b')
        self.assertInResponse(u'OK')

    def test_move_closed_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'move "1:3" "0"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'b')
        self.assertEqual(tracks[1].name, 'c')
        self.assertEqual(tracks[2].name, 'a')
        self.assertEqual(tracks[3].name, 'd')
        self.assertEqual(tracks[4].name, 'e')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')

    def test_moveid(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'moveid "4" "2"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'a')
        self.assertEqual(tracks[1].name, 'b')
        self.assertEqual(tracks[2].name, 'e')
        self.assertEqual(tracks[3].name, 'c')
        self.assertEqual(tracks[4].name, 'd')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')

    def test_playlist_returns_same_as_playlistinfo(self):
        playlist_response = self.sendRequest(u'playlist')
        playlistinfo_response = self.sendRequest(u'playlistinfo')
        self.assertEqual(playlist_response, playlistinfo_response)

    def test_playlistfind(self):
        self.sendRequest(u'playlistfind "tag" "needle"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_playlistfind_by_filename_not_in_current_playlist(self):
        self.sendRequest(u'playlistfind "filename" "file:///dev/null"')
        self.assertEqualResponse(u'OK')

    def test_playlistfind_by_filename_without_quotes(self):
        self.sendRequest(u'playlistfind filename "file:///dev/null"')
        self.assertEqualResponse(u'OK')

    def test_playlistfind_by_filename_in_current_playlist(self):
        self.backend.current_playlist.append([
            Track(uri='file:///exists')])

        self.sendRequest( u'playlistfind filename "file:///exists"')
        self.assertInResponse(u'file: file:///exists')
        self.assertInResponse(u'Id: 0')
        self.assertInResponse(u'Pos: 0')
        self.assertInResponse(u'OK')

    def test_playlistid_without_songid(self):
        self.backend.current_playlist.append([Track(name='a'), Track(name='b')])

        self.sendRequest(u'playlistid')
        self.assertInResponse(u'Title: a')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'OK')

    def test_playlistid_with_songid(self):
        self.backend.current_playlist.append([Track(name='a'), Track(name='b')])

        self.sendRequest(u'playlistid "1"')
        self.assertNotInResponse(u'Title: a')
        self.assertNotInResponse(u'Id: 0')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'Id: 1')
        self.assertInResponse(u'OK')

    def test_playlistid_with_not_existing_songid_fails(self):
        self.backend.current_playlist.append([Track(name='a'), Track(name='b')])

        self.sendRequest(u'playlistid "25"')
        self.assertEqualResponse(u'ACK [50@0] {playlistid} No such song')

    def test_playlistinfo_without_songpos_or_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'playlistinfo')
        self.assertInResponse(u'Title: a')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'Title: d')
        self.assertInResponse(u'Title: e')
        self.assertInResponse(u'Title: f')
        self.assertInResponse(u'OK')

    def test_playlistinfo_with_songpos(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'playlistinfo "4"')
        self.assertNotInResponse(u'Title: a')
        self.assertNotInResponse(u'Title: b')
        self.assertNotInResponse(u'Title: c')
        self.assertNotInResponse(u'Title: d')
        self.assertInResponse(u'Title: e')
        self.assertNotInResponse(u'Title: f')
        self.assertInResponse(u'OK')

    def test_playlistinfo_with_negative_songpos_same_as_playlistinfo(self):
        response1 = self.sendRequest(u'playlistinfo "-1"')
        response2 = self.sendRequest(u'playlistinfo')
        self.assertEqual(response1, response2)

    def test_playlistinfo_with_open_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'playlistinfo "2:"')
        self.assertNotInResponse(u'Title: a')
        self.assertNotInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'Title: d')
        self.assertInResponse(u'Title: e')
        self.assertInResponse(u'Title: f')
        self.assertInResponse(u'OK')

    def test_playlistinfo_with_closed_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'playlistinfo "2:4"')
        self.assertNotInResponse(u'Title: a')
        self.assertNotInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'Title: d')
        self.assertNotInResponse(u'Title: e')
        self.assertNotInResponse(u'Title: f')
        self.assertInResponse(u'OK')

    def test_playlistinfo_with_too_high_start_of_range_returns_arg_error(self):
        self.sendRequest(u'playlistinfo "10:20"')
        self.assertEqualResponse(u'ACK [2@0] {playlistinfo} Bad song index')

    def test_playlistinfo_with_too_high_end_of_range_returns_ok(self):
        self.sendRequest(u'playlistinfo "0:20"')
        self.assertInResponse(u'OK')

    def test_playlistsearch(self):
        self.sendRequest( u'playlistsearch "any" "needle"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_playlistsearch_without_quotes(self):
        self.sendRequest(u'playlistsearch any "needle"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_plchanges(self):
        self.backend.current_playlist.append(
            [Track(name='a'), Track(name='b'), Track(name='c')])

        self.sendRequest(u'plchanges "0"')
        self.assertInResponse(u'Title: a')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'OK')

    def test_plchanges_with_minus_one_returns_entire_playlist(self):
        self.backend.current_playlist.append(
            [Track(name='a'), Track(name='b'), Track(name='c')])

        self.sendRequest(u'plchanges "-1"')
        self.assertInResponse(u'Title: a')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'OK')

    def test_plchanges_without_quotes_works(self):
        self.backend.current_playlist.append(
            [Track(name='a'), Track(name='b'), Track(name='c')])

        self.sendRequest(u'plchanges 0')
        self.assertInResponse(u'Title: a')
        self.assertInResponse(u'Title: b')
        self.assertInResponse(u'Title: c')
        self.assertInResponse(u'OK')

    def test_plchangesposid(self):
        self.backend.current_playlist.append([Track(), Track(), Track()])

        self.sendRequest(u'plchangesposid "0"')
        cp_tracks = self.backend.current_playlist.cp_tracks.get()
        self.assertInResponse(u'cpos: 0')
        self.assertInResponse(u'Id: %d' % cp_tracks[0][0])
        self.assertInResponse(u'cpos: 2')
        self.assertInResponse(u'Id: %d' % cp_tracks[1][0])
        self.assertInResponse(u'cpos: 2')
        self.assertInResponse(u'Id: %d' % cp_tracks[2][0])
        self.assertInResponse(u'OK')

    def test_shuffle_without_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])
        version = self.backend.current_playlist.version.get()

        self.sendRequest(u'shuffle')
        self.assert_(version < self.backend.current_playlist.version.get())
        self.assertInResponse(u'OK')

    def test_shuffle_with_open_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])
        version = self.backend.current_playlist.version.get()

        self.sendRequest(u'shuffle "4:"')
        self.assert_(version < self.backend.current_playlist.version.get())
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'a')
        self.assertEqual(tracks[1].name, 'b')
        self.assertEqual(tracks[2].name, 'c')
        self.assertEqual(tracks[3].name, 'd')
        self.assertInResponse(u'OK')

    def test_shuffle_with_closed_range(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])
        version = self.backend.current_playlist.version.get()

        self.sendRequest(u'shuffle "1:3"')
        self.assert_(version < self.backend.current_playlist.version.get())
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'a')
        self.assertEqual(tracks[3].name, 'd')
        self.assertEqual(tracks[4].name, 'e')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')

    def test_swap(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'swap "1" "4"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'a')
        self.assertEqual(tracks[1].name, 'e')
        self.assertEqual(tracks[2].name, 'c')
        self.assertEqual(tracks[3].name, 'd')
        self.assertEqual(tracks[4].name, 'b')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')

    def test_swapid(self):
        self.backend.current_playlist.append([
            Track(name='a'), Track(name='b'), Track(name='c'),
            Track(name='d'), Track(name='e'), Track(name='f'),
        ])

        self.sendRequest(u'swapid "1" "4"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(tracks[0].name, 'a')
        self.assertEqual(tracks[1].name, 'e')
        self.assertEqual(tracks[2].name, 'c')
        self.assertEqual(tracks[3].name, 'd')
        self.assertEqual(tracks[4].name, 'b')
        self.assertEqual(tracks[5].name, 'f')
        self.assertInResponse(u'OK')



from mopidy.mixers.dummy import DummyMixer

from tests import unittest
from tests.mixers.base_test import BaseMixerTest


class DenonMixerTest(BaseMixerTest, unittest.TestCase):
    mixer_class = DummyMixer

    def test_set_volume_is_capped(self):
        self.mixer.amplification_factor = 0.5
        self.mixer.volume = 100
        self.assertEquals(self.mixer._volume, 50)

    def test_get_volume_does_not_show_that_the_volume_is_capped(self):
        self.mixer.amplification_factor = 0.5
        self.mixer._volume = 50
        self.assertEquals(self.mixer.volume, 100)



from mopidy.frontends.mpd.protocol import handle_request
from mopidy.frontends.mpd.exceptions import MpdUnknownCommand

@handle_request(r'^command_list_begin$')
def command_list_begin(context):
    """
    *musicpd.org, command list section:*

        To facilitate faster adding of files etc. you can pass a list of
        commands all at once using a command list. The command list begins
        with ``command_list_begin`` or ``command_list_ok_begin`` and ends
        with ``command_list_end``.

        It does not execute any commands until the list has ended. The
        return value is whatever the return for a list of commands is. On
        success for all commands, ``OK`` is returned. If a command fails,
        no more commands are executed and the appropriate ``ACK`` error is
        returned. If ``command_list_ok_begin`` is used, ``list_OK`` is
        returned for each successful command executed in the command list.
    """
    context.dispatcher.command_list = []
    context.dispatcher.command_list_ok = False

@handle_request(r'^command_list_end$')
def command_list_end(context):
    """See :meth:`command_list_begin()`."""
    if context.dispatcher.command_list is False:
        # Test for False exactly, and not e.g. empty list
        raise MpdUnknownCommand(command='command_list_end')
    (command_list, context.dispatcher.command_list) = (
        context.dispatcher.command_list, False)
    (command_list_ok, context.dispatcher.command_list_ok) = (
        context.dispatcher.command_list_ok, False)
    command_list_response = []
    for index, command in enumerate(command_list):
        response = context.dispatcher.handle_request(
            command, current_command_list_index=index)
        command_list_response.extend(response)
        if (command_list_response and
                command_list_response[-1].startswith(u'ACK')):
            return command_list_response
        if command_list_ok:
            command_list_response.append(u'list_OK')
    return command_list_response

@handle_request(r'^command_list_ok_begin$')
def command_list_ok_begin(context):
    """See :meth:`command_list_begin()`."""
    context.dispatcher.command_list = []
    context.dispatcher.command_list_ok = True



import glob
import glib
import logging
import os
import shutil

from pykka.actor import ThreadingActor
from pykka.registry import ActorRegistry

from mopidy import settings, DATA_PATH
from mopidy.backends.base import (Backend, CurrentPlaylistController,
    LibraryController, BaseLibraryProvider, PlaybackController,
    BasePlaybackProvider, StoredPlaylistsController,
    BaseStoredPlaylistsProvider)
from mopidy.models import Playlist, Track, Album
from mopidy.gstreamer import GStreamer

from .translator import parse_m3u, parse_mpd_tag_cache

logger = logging.getLogger(u'mopidy.backends.local')

DEFAULT_PLAYLIST_PATH = os.path.join(DATA_PATH, 'playlists')
DEFAULT_TAG_CACHE_FILE = os.path.join(DATA_PATH, 'tag_cache')
DEFAULT_MUSIC_PATH = glib.get_user_special_dir(glib.USER_DIRECTORY_MUSIC)

if not DEFAULT_MUSIC_PATH or DEFAULT_MUSIC_PATH == os.path.expanduser(u'~'):
    DEFAULT_MUSIC_PATH = os.path.expanduser(u'~/music')


class LocalBackend(ThreadingActor, Backend):
    """
    A backend for playing music from a local music archive.

    **Issues:** https://github.com/mopidy/mopidy/issues?labels=backend-local

    **Dependencies:**

    - None

    **Settings:**

    - :attr:`mopidy.settings.LOCAL_MUSIC_PATH`
    - :attr:`mopidy.settings.LOCAL_PLAYLIST_PATH`
    - :attr:`mopidy.settings.LOCAL_TAG_CACHE_FILE`
    """

    def __init__(self, *args, **kwargs):
        super(LocalBackend, self).__init__(*args, **kwargs)

        self.current_playlist = CurrentPlaylistController(backend=self)

        library_provider = LocalLibraryProvider(backend=self)
        self.library = LibraryController(backend=self,
            provider=library_provider)

        playback_provider = LocalPlaybackProvider(backend=self)
        self.playback = LocalPlaybackController(backend=self,
            provider=playback_provider)

        stored_playlists_provider = LocalStoredPlaylistsProvider(backend=self)
        self.stored_playlists = StoredPlaylistsController(backend=self,
            provider=stored_playlists_provider)

        self.uri_schemes = [u'file']

        self.gstreamer = None

    def on_start(self):
        gstreamer_refs = ActorRegistry.get_by_class(GStreamer)
        assert len(gstreamer_refs) == 1, \
            'Expected exactly one running GStreamer.'
        self.gstreamer = gstreamer_refs[0].proxy()


class LocalPlaybackController(PlaybackController):
    def __init__(self, *args, **kwargs):
        super(LocalPlaybackController, self).__init__(*args, **kwargs)

        # XXX Why do we call stop()? Is it to set GStreamer state to 'READY'?
        self.stop()

    @property
    def time_position(self):
        return self.backend.gstreamer.get_position().get()


class LocalPlaybackProvider(BasePlaybackProvider):
    def pause(self):
        return self.backend.gstreamer.pause_playback().get()

    def play(self, track):
        self.backend.gstreamer.prepare_change()
        self.backend.gstreamer.set_uri(track.uri).get()
        return self.backend.gstreamer.start_playback().get()

    def resume(self):
        return self.backend.gstreamer.start_playback().get()

    def seek(self, time_position):
        return self.backend.gstreamer.set_position(time_position).get()

    def stop(self):
        return self.backend.gstreamer.stop_playback().get()


class LocalStoredPlaylistsProvider(BaseStoredPlaylistsProvider):
    def __init__(self, *args, **kwargs):
        super(LocalStoredPlaylistsProvider, self).__init__(*args, **kwargs)
        self._folder = settings.LOCAL_PLAYLIST_PATH or DEFAULT_PLAYLIST_PATH
        self.refresh()

    def lookup(self, uri):
        pass # TODO

    def refresh(self):
        playlists = []

        logger.info('Loading playlists from %s', self._folder)

        for m3u in glob.glob(os.path.join(self._folder, '*.m3u')):
            name = os.path.basename(m3u)[:len('.m3u')]
            tracks = []
            for uri in parse_m3u(m3u):
                try:
                    tracks.append(self.backend.library.lookup(uri))
                except LookupError, e:
                    logger.error('Playlist item could not be added: %s', e)
            playlist = Playlist(tracks=tracks, name=name)

            # FIXME playlist name needs better handling
            # FIXME tracks should come from lib. lookup

            playlists.append(playlist)

        self.playlists = playlists

    def create(self, name):
        playlist = Playlist(name=name)
        self.save(playlist)
        return playlist

    def delete(self, playlist):
        if playlist not in self._playlists:
            return

        self._playlists.remove(playlist)
        filename = os.path.join(self._folder, playlist.name + '.m3u')

        if os.path.exists(filename):
            os.remove(filename)

    def rename(self, playlist, name):
        if playlist not in self._playlists:
            return

        src = os.path.join(self._folder, playlist.name + '.m3u')
        dst = os.path.join(self._folder, name + '.m3u')

        renamed = playlist.copy(name=name)
        index = self._playlists.index(playlist)
        self._playlists[index] = renamed

        shutil.move(src, dst)

    def save(self, playlist):
        file_path = os.path.join(self._folder, playlist.name + '.m3u')

        # FIXME this should be a save_m3u function, not inside save
        with open(file_path, 'w') as file_handle:
            for track in playlist.tracks:
                if track.uri.startswith('file://'):
                    file_handle.write(track.uri[len('file://'):] + '\n')
                else:
                    file_handle.write(track.uri + '\n')

        self._playlists.append(playlist)


class LocalLibraryProvider(BaseLibraryProvider):
    def __init__(self, *args, **kwargs):
        super(LocalLibraryProvider, self).__init__(*args, **kwargs)
        self._uri_mapping = {}
        self.refresh()

    def refresh(self, uri=None):
        tag_cache = settings.LOCAL_TAG_CACHE_FILE or DEFAULT_TAG_CACHE_FILE
        music_folder = settings.LOCAL_MUSIC_PATH or DEFAULT_MUSIC_PATH

        tracks = parse_mpd_tag_cache(tag_cache, music_folder)

        logger.info('Loading tracks in %s from %s', music_folder, tag_cache)

        for track in tracks:
            self._uri_mapping[track.uri] = track

    def lookup(self, uri):
        try:
            return self._uri_mapping[uri]
        except KeyError:
            raise LookupError('%s not found.' % uri)

    def find_exact(self, **query):
        self._validate_query(query)
        result_tracks = self._uri_mapping.values()

        for (field, values) in query.iteritems():
            if not hasattr(values, '__iter__'):
                values = [values]
            # FIXME this is bound to be slow for large libraries
            for value in values:
                q = value.strip()

                track_filter = lambda t: q == t.name
                album_filter = lambda t: q == getattr(t, 'album', Album()).name
                artist_filter = lambda t: filter(
                    lambda a: q == a.name, t.artists)
                uri_filter = lambda t: q == t.uri
                any_filter = lambda t: (track_filter(t) or album_filter(t) or
                    artist_filter(t) or uri_filter(t))

                if field == 'track':
                    result_tracks = filter(track_filter, result_tracks)
                elif field == 'album':
                    result_tracks = filter(album_filter, result_tracks)
                elif field == 'artist':
                    result_tracks = filter(artist_filter, result_tracks)
                elif field == 'uri':
                    result_tracks = filter(uri_filter, result_tracks)
                elif field == 'any':
                    result_tracks = filter(any_filter, result_tracks)
                else:
                    raise LookupError('Invalid lookup field: %s' % field)
        return Playlist(tracks=result_tracks)

    def search(self, **query):
        self._validate_query(query)
        result_tracks = self._uri_mapping.values()

        for (field, values) in query.iteritems():
            if not hasattr(values, '__iter__'):
                values = [values]
            # FIXME this is bound to be slow for large libraries
            for value in values:
                q = value.strip().lower()

                track_filter  = lambda t: q in t.name.lower()
                album_filter = lambda t: q in getattr(
                    t, 'album', Album()).name.lower()
                artist_filter = lambda t: filter(
                    lambda a: q in a.name.lower(), t.artists)
                uri_filter = lambda t: q in t.uri.lower()
                any_filter = lambda t: track_filter(t) or album_filter(t) or \
                    artist_filter(t) or uri_filter(t)

                if field == 'track':
                    result_tracks = filter(track_filter, result_tracks)
                elif field == 'album':
                    result_tracks = filter(album_filter, result_tracks)
                elif field == 'artist':
                    result_tracks = filter(artist_filter, result_tracks)
                elif field == 'uri':
                    result_tracks = filter(uri_filter, result_tracks)
                elif field == 'any':
                    result_tracks = filter(any_filter, result_tracks)
                else:
                    raise LookupError('Invalid lookup field: %s' % field)
        return Playlist(tracks=result_tracks)

    def _validate_query(self, query):
        for (_, values) in query.iteritems():
            if not values:
                raise LookupError('Missing query')
            for value in values:
                if not value:
                    raise LookupError('Missing query')



from pykka.actor import ThreadingActor

from mopidy.backends.base import (Backend, CurrentPlaylistController,
    PlaybackController, BasePlaybackProvider, LibraryController,
    BaseLibraryProvider, StoredPlaylistsController,
    BaseStoredPlaylistsProvider)
from mopidy.models import Playlist


class DummyBackend(ThreadingActor, Backend):
    """
    A backend which implements the backend API in the simplest way possible.
    Used in tests of the frontends.

    Handles URIs starting with ``dummy:``.
    """

    def __init__(self, *args, **kwargs):
        super(DummyBackend, self).__init__(*args, **kwargs)

        self.current_playlist = CurrentPlaylistController(backend=self)

        library_provider = DummyLibraryProvider(backend=self)
        self.library = LibraryController(backend=self,
            provider=library_provider)

        playback_provider = DummyPlaybackProvider(backend=self)
        self.playback = PlaybackController(backend=self,
            provider=playback_provider)

        stored_playlists_provider = DummyStoredPlaylistsProvider(backend=self)
        self.stored_playlists = StoredPlaylistsController(backend=self,
            provider=stored_playlists_provider)

        self.uri_schemes = [u'dummy']


class DummyLibraryProvider(BaseLibraryProvider):
    def __init__(self, *args, **kwargs):
        super(DummyLibraryProvider, self).__init__(*args, **kwargs)
        self.dummy_library = []

    def find_exact(self, **query):
        return Playlist()

    def lookup(self, uri):
        matches = filter(lambda t: uri == t.uri, self.dummy_library)
        if matches:
            return matches[0]

    def refresh(self, uri=None):
        pass

    def search(self, **query):
        return Playlist()


class DummyPlaybackProvider(BasePlaybackProvider):
    def pause(self):
        return True

    def play(self, track):
        """Pass None as track to force failure"""
        return track is not None

    def resume(self):
        return True

    def seek(self, time_position):
        return True

    def stop(self):
        return True


class DummyStoredPlaylistsProvider(BaseStoredPlaylistsProvider):
    def create(self, name):
        playlist = Playlist(name=name)
        self._playlists.append(playlist)
        return playlist

    def delete(self, playlist):
        self._playlists.remove(playlist)

    def lookup(self, uri):
        return filter(lambda p: p.uri == uri, self._playlists)

    def refresh(self):
        pass

    def rename(self, playlist, new_name):
        self._playlists[self._playlists.index(playlist)] = \
            playlist.copy(name=new_name)

    def save(self, playlist):
        self._playlists.append(playlist)



import logging
import serial

from pykka.actor import ThreadingActor

from mopidy import settings
from mopidy.mixers.base import BaseMixer

logger = logging.getLogger('mopidy.mixers.nad')

class NadMixer(ThreadingActor, BaseMixer):
    """
    Mixer for controlling NAD amplifiers and receivers using the NAD RS-232
    protocol.

    The NAD mixer was created using a NAD C 355BEE amplifier, but should also
    work with other NAD amplifiers supporting the same RS-232 protocol (v2.x).
    The C 355BEE does not give you access to the current volume. It only
    supports increasing or decreasing the volume one step at the time. Other
    NAD amplifiers may support more advanced volume adjustment than what is
    currently used by this mixer.

    Sadly, this means that if you use the remote control to change the volume
    on the amplifier, Mopidy will no longer report the correct volume.

    **Dependencies**

    - pyserial (python-serial on Debian/Ubuntu)

    **Settings**

    - :attr:`mopidy.settings.MIXER_EXT_PORT` -- Example: ``/dev/ttyUSB0``
    - :attr:`mopidy.settings.MIXER_EXT_SOURCE` -- Example: ``Aux``
    - :attr:`mopidy.settings.MIXER_EXT_SPEAKERS_A` -- Example: ``On``
    - :attr:`mopidy.settings.MIXER_EXT_SPEAKERS_B` -- Example: ``Off``

    """

    def __init__(self):
        self._volume_cache = None
        self._nad_talker = NadTalker.start().proxy()

    def get_volume(self):
        return self._volume_cache

    def set_volume(self, volume):
        self._volume_cache = volume
        self._nad_talker.set_volume(volume)


class NadTalker(ThreadingActor):
    """
    Independent process which does the communication with the NAD device.

    Since the communication is done in an independent process, Mopidy won't
    block other requests while doing rather time consuming work like
    calibrating the NAD device's volume.
    """

    # Timeout in seconds used for read/write operations.
    # If you set the timeout too low, the reads will never get complete
    # confirmations and calibration will decrease volume forever. If you set
    # the timeout too high, stuff takes more time. 0.2s seems like a good value
    # for NAD C 355BEE.
    TIMEOUT = 0.2

    # Number of volume levels the device supports. 40 for NAD C 355BEE.
    VOLUME_LEVELS = 40

    # Volume in range 0..VOLUME_LEVELS. :class:`None` before calibration.
    _nad_volume = None

    def __init__(self):
        self._device = None

    def on_start(self):
        self._open_connection()
        self._set_device_to_known_state()

    def _open_connection(self):
        # Opens serial connection to the device.
        # Communication settings: 115200 bps 8N1
        logger.info(u'Connecting to serial device "%s"',
            settings.MIXER_EXT_PORT)
        self._device = serial.Serial(port=settings.MIXER_EXT_PORT,
            baudrate=115200, timeout=self.TIMEOUT)
        self._get_device_model()

    def _set_device_to_known_state(self):
        self._power_device_on()
        self._select_speakers()
        self._select_input_source()
        self._unmute()
        self._calibrate_volume()

    def _get_device_model(self):
        model = self._ask_device('Main.Model')
        logger.info(u'Connected to device of model "%s"', model)
        return model

    def _power_device_on(self):
        while self._ask_device('Main.Power') != 'On':
            logger.info(u'Powering device on')
            self._command_device('Main.Power', 'On')

    def _select_speakers(self):
        if settings.MIXER_EXT_SPEAKERS_A is not None:
            while (self._ask_device('Main.SpeakerA')
                    != settings.MIXER_EXT_SPEAKERS_A):
                logger.info(u'Setting speakers A "%s"',
                    settings.MIXER_EXT_SPEAKERS_A)
                self._command_device('Main.SpeakerA',
                    settings.MIXER_EXT_SPEAKERS_A)
        if settings.MIXER_EXT_SPEAKERS_B is not None:
            while (self._ask_device('Main.SpeakerB') !=
                    settings.MIXER_EXT_SPEAKERS_B):
                logger.info(u'Setting speakers B "%s"',
                    settings.MIXER_EXT_SPEAKERS_B)
                self._command_device('Main.SpeakerB',
                    settings.MIXER_EXT_SPEAKERS_B)

    def _select_input_source(self):
        if settings.MIXER_EXT_SOURCE is not None:
            while self._ask_device('Main.Source') != settings.MIXER_EXT_SOURCE:
                logger.info(u'Selecting input source "%s"',
                    settings.MIXER_EXT_SOURCE)
                self._command_device('Main.Source', settings.MIXER_EXT_SOURCE)

    def _unmute(self):
        while self._ask_device('Main.Mute') != 'Off':
            logger.info(u'Unmuting device')
            self._command_device('Main.Mute', 'Off')

    def _ask_device(self, key):
        self._write('%s?' % key)
        return self._readline().replace('%s=' % key, '')

    def _command_device(self, key, value):
        if type(value) == unicode:
            value = value.encode('utf-8')
        self._write('%s=%s' % (key, value))
        self._readline()

    def _calibrate_volume(self):
        # The NAD C 355BEE amplifier has 40 different volume levels. We have no
        # way of asking on which level we are. Thus, we must calibrate the
        # mixer by decreasing the volume 39 times.
        logger.info(u'Calibrating NAD amplifier')
        steps_left = self.VOLUME_LEVELS - 1
        while steps_left:
            if self._decrease_volume():
                steps_left -= 1
        self._nad_volume = 0
        logger.info(u'Done calibrating NAD amplifier')

    def set_volume(self, volume):
        # Increase or decrease the amplifier volume until it matches the given
        # target volume.
        logger.debug(u'Setting volume to %d' % volume)
        target_nad_volume = int(round(volume * self.VOLUME_LEVELS / 100.0))
        if self._nad_volume is None:
            return # Calibration needed
        while target_nad_volume > self._nad_volume:
            if self._increase_volume():
                self._nad_volume += 1
        while target_nad_volume < self._nad_volume:
            if self._decrease_volume():
                self._nad_volume -= 1

    def _increase_volume(self):
        # Increase volume. Returns :class:`True` if confirmed by device.
        self._write('Main.Volume+')
        return self._readline() == 'Main.Volume+'

    def _decrease_volume(self):
        # Decrease volume. Returns :class:`True` if confirmed by device.
        self._write('Main.Volume-')
        return self._readline() == 'Main.Volume-'

    def _write(self, data):
        # Write data to device. Prepends and appends a newline to the data, as
        # recommended by the NAD documentation.
        if not self._device.isOpen():
            self._device.open()
        self._device.write('\n%s\n' % data)
        logger.debug('Write: %s', data)

    def _readline(self):
        # Read line from device. The result is stripped for leading and
        # trailing whitespace.
        if not self._device.isOpen():
            self._device.open()
        result = self._device.readline().strip()
        if result:
            logger.debug('Read: %s', result)
        return result



import logging

from pykka.actor import ThreadingActor

from mopidy import settings
from mopidy.mixers.base import BaseMixer

logger = logging.getLogger(u'mopidy.mixers.denon')

class DenonMixer(ThreadingActor, BaseMixer):
    """
    Mixer for controlling Denon amplifiers and receivers using the RS-232
    protocol.

    The external mixer is the authoritative source for the current volume.
    This allows the user to use his remote control the volume without Mopidy
    cancelling the volume setting.

    **Dependencies**

    - pyserial (python-serial on Debian/Ubuntu)

    **Settings**

    - :attr:`mopidy.settings.MIXER_EXT_PORT` -- Example: ``/dev/ttyUSB0``
    """

    def __init__(self, *args, **kwargs):
        self._device = kwargs.get('device', None)
        self._levels = ['99'] + ["%(#)02d" % {'#': v} for v in range(0, 99)]
        self._volume = 0

    def on_start(self):
        if self._device is None:
            from serial import Serial
            self._device = Serial(port=settings.MIXER_EXT_PORT, timeout=0.2)

    def get_volume(self):
        self._ensure_open_device()
        self._device.write('MV?\r')
        vol = str(self._device.readline()[2:4])
        logger.debug(u'_get_volume() = %s' % vol)
        return self._levels.index(vol)

    def set_volume(self, volume):
        # Clamp according to Denon-spec
        if volume > 99:
            volume = 99
        self._ensure_open_device()
        self._device.write('MV%s\r'% self._levels[volume])
        vol = self._device.readline()[2:4]
        self._volume = self._levels.index(vol)

    def _ensure_open_device(self):
        if not self._device.isOpen():
            logger.debug(u'(re)connecting to Denon device')
            self._device.open()



import logging
import time

try:
    import pylast
except ImportError as import_error:
    from mopidy import OptionalDependencyError
    raise OptionalDependencyError(import_error)

from pykka.actor import ThreadingActor

from mopidy import settings, SettingsError
from mopidy.listeners import BackendListener

logger = logging.getLogger('mopidy.frontends.lastfm')

API_KEY = '2236babefa8ebb3d93ea467560d00d04'
API_SECRET = '94d9a09c0cd5be955c4afaeaffcaefcd'

class LastfmFrontend(ThreadingActor, BackendListener):
    """
    Frontend which scrobbles the music you play to your `Last.fm
    <http://www.last.fm>`_ profile.

    .. note::

        This frontend requires a free user account at Last.fm.

    **Dependencies:**

    - `pylast <http://code.google.com/p/pylast/>`_ >= 0.5.7

    **Settings:**

    - :attr:`mopidy.settings.LASTFM_USERNAME`
    - :attr:`mopidy.settings.LASTFM_PASSWORD`
    """

    def __init__(self):
        self.lastfm = None
        self.last_start_time = None

    def on_start(self):
        try:
            username = settings.LASTFM_USERNAME
            password_hash = pylast.md5(settings.LASTFM_PASSWORD)
            self.lastfm = pylast.LastFMNetwork(
                api_key=API_KEY, api_secret=API_SECRET,
                username=username, password_hash=password_hash)
            logger.info(u'Connected to Last.fm')
        except SettingsError as e:
            logger.info(u'Last.fm scrobbler not started')
            logger.debug(u'Last.fm settings error: %s', e)
            self.stop()
        except (pylast.NetworkError, pylast.MalformedResponseError,
                pylast.WSError) as e:
            logger.error(u'Error during Last.fm setup: %s', e)
            self.stop()

    def track_playback_started(self, track):
        artists = ', '.join([a.name for a in track.artists])
        duration = track.length and track.length // 1000 or 0
        self.last_start_time = int(time.time())
        logger.debug(u'Now playing track: %s - %s', artists, track.name)
        try:
            self.lastfm.update_now_playing(
                artists,
                (track.name or ''),
                album=(track.album and track.album.name or ''),
                duration=str(duration),
                track_number=str(track.track_no),
                mbid=(track.musicbrainz_id or ''))
        except (pylast.ScrobblingError, pylast.NetworkError,
                pylast.MalformedResponseError, pylast.WSError) as e:
            logger.warning(u'Error submitting playing track to Last.fm: %s', e)

    def track_playback_ended(self, track, time_position):
        artists = ', '.join([a.name for a in track.artists])
        duration = track.length and track.length // 1000 or 0
        time_position = time_position // 1000
        if duration < 30:
            logger.debug(u'Track too short to scrobble. (30s)')
            return
        if time_position < duration // 2 and time_position < 240:
            logger.debug(
                u'Track not played long enough to scrobble. (50% or 240s)')
            return
        if self.last_start_time is None:
            self.last_start_time = int(time.time()) - duration
        logger.debug(u'Scrobbling track: %s - %s', artists, track.name)
        try:
            self.lastfm.scrobble(
                artists,
                (track.name or ''),
                str(self.last_start_time),
                album=(track.album and track.album.name or ''),
                track_number=str(track.track_no),
                duration=str(duration),
                mbid=(track.musicbrainz_id or ''))
        except (pylast.ScrobblingError, pylast.NetworkError,
                pylast.MalformedResponseError, pylast.WSError) as e:
            logger.warning(u'Error submitting played track to Last.fm: %s', e)



from mock import patch

from mopidy.frontends.mpd.protocol.status import SUBSYSTEMS

from tests.frontends.mpd import protocol


class IdleHandlerTest(protocol.BaseTestCase):
    def idleEvent(self, subsystem):
        self.session.on_idle(subsystem)

    def assertEqualEvents(self, events):
        self.assertEqual(set(events), self.context.events)

    def assertEqualSubscriptions(self, events):
        self.assertEqual(set(events), self.context.subscriptions)

    def assertNoEvents(self):
        self.assertEqualEvents([])

    def assertNoSubscriptions(self):
        self.assertEqualSubscriptions([])

    def test_base_state(self):
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertNoResponse()

    def test_idle(self):
        self.sendRequest(u'idle')
        self.assertEqualSubscriptions(SUBSYSTEMS)
        self.assertNoEvents()
        self.assertNoResponse()

    def test_idle_disables_timeout(self):
        self.sendRequest(u'idle')
        self.connection.disable_timeout.assert_called_once_with()

    def test_noidle(self):
        self.sendRequest(u'noidle')
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertNoResponse()

    def test_idle_player(self):
        self.sendRequest(u'idle player')
        self.assertEqualSubscriptions(['player'])
        self.assertNoEvents()
        self.assertNoResponse()

    def test_idle_player_playlist(self):
        self.sendRequest(u'idle player playlist')
        self.assertEqualSubscriptions(['player', 'playlist'])
        self.assertNoEvents()
        self.assertNoResponse()

    def test_idle_then_noidle(self):
        self.sendRequest(u'idle')
        self.sendRequest(u'noidle')
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertOnceInResponse(u'OK')

    def test_idle_then_noidle_enables_timeout(self):
        self.sendRequest(u'idle')
        self.sendRequest(u'noidle')
        self.connection.enable_timeout.assert_called_once_with()

    def test_idle_then_play(self):
        with patch.object(self.session, 'stop') as stop_mock:
            self.sendRequest(u'idle')
            self.sendRequest(u'play')
            stop_mock.assert_called_once_with()

    def test_idle_then_idle(self):
        with patch.object(self.session, 'stop') as stop_mock:
            self.sendRequest(u'idle')
            self.sendRequest(u'idle')
            stop_mock.assert_called_once_with()

    def test_idle_player_then_play(self):
        with patch.object(self.session, 'stop') as stop_mock:
            self.sendRequest(u'idle player')
            self.sendRequest(u'play')
            stop_mock.assert_called_once_with()

    def test_idle_then_player(self):
        self.sendRequest(u'idle')
        self.idleEvent(u'player')
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertOnceInResponse(u'changed: player')
        self.assertOnceInResponse(u'OK')

    def test_idle_player_then_event_player(self):
        self.sendRequest(u'idle player')
        self.idleEvent(u'player')
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertOnceInResponse(u'changed: player')
        self.assertOnceInResponse(u'OK')

    def test_idle_player_then_noidle(self):
        self.sendRequest(u'idle player')
        self.sendRequest(u'noidle')
        self.assertNoSubscriptions()
        self.assertNoEvents()
        self.assertOnceInResponse(u'OK')

    def test_idle_player_playlist_then_noidle(self):
        self.sendRequest(u'idle player playlist')
        self.sendRequest(u'noidle')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'OK')

    def test_idle_player_playlist_then_player(self):
        self.sendRequest(u'idle player playlist')
        self.idleEvent(u'player')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'changed: player')
        self.assertNotInResponse(u'changed: playlist')
        self.assertOnceInResponse(u'OK')

    def test_idle_playlist_then_player(self):
        self.sendRequest(u'idle playlist')
        self.idleEvent(u'player')
        self.assertEqualEvents(['player'])
        self.assertEqualSubscriptions(['playlist'])
        self.assertNoResponse()

    def test_idle_playlist_then_player_then_playlist(self):
        self.sendRequest(u'idle playlist')
        self.idleEvent(u'player')
        self.idleEvent(u'playlist')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertNotInResponse(u'changed: player')
        self.assertOnceInResponse(u'changed: playlist')
        self.assertOnceInResponse(u'OK')

    def test_player(self):
        self.idleEvent(u'player')
        self.assertEqualEvents(['player'])
        self.assertNoSubscriptions()
        self.assertNoResponse()

    def test_player_then_idle_player(self):
        self.idleEvent(u'player')
        self.sendRequest(u'idle player')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'changed: player')
        self.assertNotInResponse(u'changed: playlist')
        self.assertOnceInResponse(u'OK')

    def test_player_then_playlist(self):
        self.idleEvent(u'player')
        self.idleEvent(u'playlist')
        self.assertEqualEvents(['player', 'playlist'])
        self.assertNoSubscriptions()
        self.assertNoResponse()

    def test_player_then_idle(self):
        self.idleEvent(u'player')
        self.sendRequest(u'idle')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'changed: player')
        self.assertOnceInResponse(u'OK')

    def test_player_then_playlist_then_idle(self):
        self.idleEvent(u'player')
        self.idleEvent(u'playlist')
        self.sendRequest(u'idle')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'changed: player')
        self.assertOnceInResponse(u'changed: playlist')
        self.assertOnceInResponse(u'OK')

    def test_player_then_idle_playlist(self):
        self.idleEvent(u'player')
        self.sendRequest(u'idle playlist')
        self.assertEqualEvents(['player'])
        self.assertEqualSubscriptions(['playlist'])
        self.assertNoResponse()

    def test_player_then_idle_playlist_then_noidle(self):
        self.idleEvent(u'player')
        self.sendRequest(u'idle playlist')
        self.sendRequest(u'noidle')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertOnceInResponse(u'OK')

    def test_player_then_playlist_then_idle_playlist(self):
        self.idleEvent(u'player')
        self.idleEvent(u'playlist')
        self.sendRequest(u'idle playlist')
        self.assertNoEvents()
        self.assertNoSubscriptions()
        self.assertNotInResponse(u'changed: player')
        self.assertOnceInResponse(u'changed: playlist')
        self.assertOnceInResponse(u'OK')



import platform
import sys
if not (2, 6) <= sys.version_info < (3,):
    sys.exit(u'Mopidy requires Python >= 2.6, < 3')

import glib
import os

from subprocess import PIPE, Popen

VERSION = (0, 6, 1)

DATA_PATH = os.path.join(glib.get_user_data_dir(), 'mopidy')
CACHE_PATH = os.path.join(glib.get_user_cache_dir(), 'mopidy')
SETTINGS_PATH = os.path.join(glib.get_user_config_dir(), 'mopidy')
SETTINGS_FILE = os.path.join(SETTINGS_PATH, 'settings.py')

def get_version():
    try:
        return get_git_version()
    except EnvironmentError:
        return get_plain_version()

def get_git_version():
    process = Popen(['git', 'describe'], stdout=PIPE, stderr=PIPE)
    if process.wait() != 0:
        raise EnvironmentError('Execution of "git describe" failed')
    version = process.stdout.read().strip()
    if version.startswith('v'):
        version = version[1:]
    return version

def get_plain_version():
    return '.'.join(map(str, VERSION))

def get_platform():
    return platform.platform()

def get_python():
    implementation = platform.python_implementation()
    version = platform.python_version()
    return u' '.join([implementation, version])

class MopidyException(Exception):
    def __init__(self, message, *args, **kwargs):
        super(MopidyException, self).__init__(message, *args, **kwargs)
        self._message = message

    @property
    def message(self):
        """Reimplement message field that was deprecated in Python 2.6"""
        return self._message

    @message.setter
    def message(self, message):
        self._message = message

class SettingsError(MopidyException):
    pass

class OptionalDependencyError(MopidyException):
    pass

from mopidy import settings as default_settings_module
from mopidy.utils.settings import SettingsProxy
settings = SettingsProxy(default_settings_module)



from tests.frontends.mpd import protocol


class AudioOutputHandlerTest(protocol.BaseTestCase):
    def test_enableoutput(self):
        self.sendRequest(u'enableoutput "0"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_disableoutput(self):
        self.sendRequest(u'disableoutput "0"')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_outputs(self):
        self.sendRequest(u'outputs')
        self.assertInResponse(u'outputid: 0')
        self.assertInResponse(u'outputname: None')
        self.assertInResponse(u'outputenabled: 1')
        self.assertInResponse(u'OK')
